@@ -1,5 +1,3 @@
-#!/usr/bin/env python2
-# -*- coding: utf-8 -*-
 #
 # Copyright 2002-2014 Zuza Software Foundation
 #
@@ -33,9 +31,13 @@ from translate.storage import po, properties
 logger = logging.getLogger(__name__)
 
 
+class DiscardUnit(ValueError):
+    pass
+
+
 class prop2po:
-    """convert a .properties file to a .po file for handling the
-    translation."""
+    """convert a .properties file to a .po file for handling the translation.
+    """
 
     def __init__(self, personality="java", blankmsgstr=False,
                  duplicatestyle="msgctxt"):
@@ -51,8 +53,8 @@ class prop2po:
         thetargetfile = po.pofile()
         if self.personality in ("mozilla", "skype"):
             targetheader = thetargetfile.init_headers(
-                    x_accelerator_marker="&",
-                    x_merge_on="location",
+                x_accelerator_marker="&",
+                x_merge_on="location",
             )
         else:
             targetheader = thetargetfile.header()
@@ -66,12 +68,12 @@ class prop2po:
         appendedheader = False
         waitingcomments = []
         for propunit in thepropfile.units:
-            pounit = self.convertpropunit(thepropfile, propunit, "developer")
+            try:
+                pounit = self.convertpropunit(thepropfile, propunit, "developer")
+            except DiscardUnit:
+                continue
             if pounit is None:
                 waitingcomments.extend(propunit.comments)
-            # FIXME the storage class should not be creating blank units
-            if pounit is "discard":
-                continue
             if not appendedheader:
                 if propunit.isblank():
                     targetheader.addnote("\n".join(waitingcomments).rstrip(),
@@ -86,6 +88,8 @@ class prop2po:
                 thetargetfile.addunit(pounit)
         if self.personality == "gaia":
             thetargetfile = self.fold_gaia_plurals(thetargetfile)
+        elif self.personality == "gwt":
+            thetargetfile = self.fold_gwt_plurals(thetargetfile)
         thetargetfile.removeduplicates(self.duplicatestyle)
         return thetargetfile
 
@@ -94,8 +98,8 @@ class prop2po:
         thetargetfile = po.pofile()
         if self.personality in ("mozilla", "skype"):
             targetheader = thetargetfile.init_headers(
-                    x_accelerator_marker="&",
-                    x_merge_on="location",
+                x_accelerator_marker="&",
+                x_merge_on="location",
             )
         else:
             targetheader = thetargetfile.header()
@@ -112,16 +116,16 @@ class prop2po:
         waitingcomments = []
         # loop through the original file, looking at units one by one
         for origprop in origpropfile.units:
-            origpo = self.convertpropunit(origpropfile, origprop, "developer")
+            try:
+                origpo = self.convertpropunit(origpropfile, origprop, "developer")
+            except DiscardUnit:
+                continue
             if origpo is None:
                 waitingcomments.extend(origprop.comments)
-            # FIXME the storage class should not be creating blank units
-            if origpo is "discard":
-                continue
             # handle the header case specially...
             if not appendedheader:
                 if origprop.isblank():
-                    targetheader.addnote(u"".join(waitingcomments).rstrip(),
+                    targetheader.addnote("".join(waitingcomments).rstrip(),
                                          "developer", position="prepend")
                     waitingcomments = []
                     origpo = None
@@ -131,10 +135,11 @@ class prop2po:
                 translatedprop = translatedpropfile.locationindex[origprop.name]
                 # Need to check that this comment is not a copy of the
                 # developer comments
-                translatedpo = self.convertpropunit(translatedpropfile,
-                                                    translatedprop,
-                                                    "translator")
-                if translatedpo is "discard":
+                try:
+                    translatedpo = self.convertpropunit(translatedpropfile,
+                                                        translatedprop,
+                                                        "translator")
+                except DiscardUnit:
                     continue
             else:
                 translatedpo = None
@@ -142,7 +147,7 @@ class prop2po:
             if origpo is not None:
                 if translatedpo is not None and not self.blankmsgstr:
                     origpo.target = translatedpo.source
-                origpo.addnote(u"".join(waitingcomments).rstrip(),
+                origpo.addnote("".join(waitingcomments).rstrip(),
                                "developer", position="prepend")
                 waitingcomments = []
                 thetargetfile.addunit(origpo)
@@ -151,9 +156,97 @@ class prop2po:
                              origprop.name)
         if self.personality == "gaia":
             thetargetfile = self.fold_gaia_plurals(thetargetfile)
+        elif self.personality == "gwt":
+            thetargetfile = self.fold_gwt_plurals(thetargetfile)
         thetargetfile.removeduplicates(self.duplicatestyle)
         return thetargetfile
 
+    def fold_gwt_plurals(self, postore):
+        """Fold the multiple plural units of a gwt file into a gettext plural."""
+
+        def _append_plural_unit(plural_unit, units):
+            sources = [u.source for u in units]
+            targets = [u.target for u in units]
+            # TODO: only consider the right ones for sources and targets
+            plural_unit.source = sources
+            plural_unit.target = targets
+            plural_unit.addlocation(key)
+
+        # Map GWT variants to cldr names
+        gwt2cldr = {
+            'none': 'zero',
+            'one': 'one',
+            'two': 'two',
+            'few': 'few',
+            'many': 'many',
+            '': 'other',
+        }
+
+        class Variants(object):
+            def __init__(self, unit):
+                self.unit = unit
+                self.variants = {}
+
+        from translate.lang import data
+        import re
+        regex = re.compile(r'([^\[\]]*)(?:\[(.*)\])?')
+        names = data.cldr_plural_categories
+        new_store = type(postore)()
+        plurals = {}
+        for unit in postore.units:
+            if not unit.istranslatable():
+                #TODO: reconsider: we could lose header comments here
+                continue
+            string = unit.getlocations()[0]
+            match = regex.match(string)
+            if not match:
+                logger.warn("Invalid key: %s" % (string))
+                continue
+            key = match.group(1)
+            variant = match.group(2)
+            if key not in plurals:
+                # Generate fake unit for each keys
+                new_unit = new_store.addsourceunit("fish")
+                plurals[key] = Variants(new_unit)
+
+            # No variant => other
+            if not variant:
+                variant = ""
+
+            # Translate gwt variants to cldr names
+            old_variant = variant
+            variant = gwt2cldr.get(variant)
+
+            # Some sanity checks
+            if not variant:
+                raise Exception("Variant invalid: %s" % (old_variant))
+            if variant in plurals[key].variants:
+                logger.warn("Override %s[%s]: %s by %s" % (key, variant, str(plurals[key].variants[variant]), str(unit)))
+
+            # Put the unit
+            plurals[key].variants[variant] = unit
+
+        # Rework the set
+        for key, plural in plurals.items():
+            # We should have at least "other" (no variant in GWT)
+            if "other" not in plural.variants:
+                raise Exception("Should have property %s without any variant" % (key))
+            units = []
+            for name in names:
+                if name in plural.variants:
+                    unit = plural.variants[name]
+                    unit.target = unit.source
+                    units.append(unit)
+            # Replace the sources by good ones
+            if "one" in plural.variants and len(units) > 0:
+                units[0].source = plural.variants["one"].source
+            if "other" in plural.variants and len(units) > 1:
+                units[1].source = plural.variants["other"].source
+
+            # Create the plural unit
+            _append_plural_unit(plural.unit, units)
+        return new_store
+
     def fold_gaia_plurals(self, postore):
         """Fold the multiple plural units of a gaia file into a gettext plural."""
 
@@ -169,16 +262,16 @@ class prop2po:
 
         new_store = type(postore)()
         plurals = {}
-        current_plural = u""
+        current_plural = ""
         for unit in postore.units:
             if not unit.istranslatable():
                 #TODO: reconsider: we could lose header comments here
                 continue
-            if u"plural(n)" in unit.source:
+            if "plural(n)" in unit.source:
                 if current_plural:
                     # End of a set of plural units
                     _append_plural_unit(new_store, plurals, current_plural)
-                    current_plural = u""
+                    current_plural = ""
                 # start of a set of plural units
                 location = unit.getlocations()[0]
                 current_plural = location
@@ -189,30 +282,31 @@ class prop2po:
                 location = unit.getlocations()[0]
                 if current_plural and location.startswith(current_plural):
                     plurals[current_plural].append(unit)
-                    if not '[zero]' in location:
+                    if '[zero]' not in location:
                         # We want to keep [zero] cases separately translatable
                         continue
                 elif current_plural:
                     # End of a set of plural units
                     _append_plural_unit(new_store, plurals, current_plural)
-                    current_plural = u""
+                    current_plural = ""
 
                 new_store.addunit(unit)
 
         if current_plural:
             # The file ended with a set of plural units
             _append_plural_unit(new_store, plurals, current_plural)
-            current_plural = u""
+            current_plural = ""
 
         # if everything went well, there should be nothing left in plurals
         if len(plurals) != 0:
             logger.warning("Not all plural units converted correctly:" +
-                           "\n".join(plurals.keys()))
+                           "\n".join(plurals))
         return new_store
 
     def convertunit(self, propunit, commenttype):
-        """Converts a .properties unit to a .po unit. Returns None if empty
-        or not for translation."""
+        """Converts a .properties unit to a .po unit. Returns None if empty or
+        not for translation.
+        """
         if propunit is None:
             return None
         # escape unicode
@@ -220,14 +314,14 @@ class prop2po:
         if hasattr(propunit, "comments"):
             for comment in propunit.comments:
                 if "DONT_TRANSLATE" in comment:
-                    return "discard"
-            pounit.addnote(u"".join(propunit.getnotes()).rstrip(), commenttype)
+                    raise DiscardUnit(comment)
+            pounit.addnote(propunit.getnotes().rstrip(), commenttype)
         # TODO: handle multiline msgid
         if propunit.isblank():
             return None
         pounit.addlocation(propunit.name)
         pounit.source = propunit.source
-        pounit.target = u""
+        pounit.target = ""
         return pounit
 
     def convertmixedunit(self, labelprop, accesskeyprop, commenttype):
@@ -240,14 +334,14 @@ class prop2po:
         target_unit = po.pounit(encoding="UTF-8")
         return self.mixer.mix_units(label_unit, accesskey_unit, target_unit)
 
-    def convertpropunit(self, store, unit, commenttype, mixbucket="dtd"):
+    def convertpropunit(self, store, unit, commenttype, mixbucket="properties"):
         """Converts a unit from store to a po unit, keeping track of mixed
         names along the way.
 
         ``mixbucket`` can be specified to indicate if the given unit is part of
         the template or the translated file.
         """
-        if self.personality != "mozilla":
+        if self.personality != "mozilla" and self.personality != "gwt":
             # XXX should we enable unit mixing for other personalities?
             return self.convertunit(unit, commenttype)
 
@@ -306,8 +400,9 @@ def convertmozillaprop(inputfile, outputfile, templatefile, pot=False,
 
 def convertprop(inputfile, outputfile, templatefile, personality="java",
                 pot=False, duplicatestyle="msgctxt", encoding=None):
-    """reads in inputfile using properties, converts using prop2po, writes
-    to outputfile"""
+    """reads in inputfile using properties, converts using prop2po, writes to
+    outputfile
+    """
     inputstore = properties.propfile(inputfile, personality, encoding)
     convertor = prop2po(personality=personality, blankmsgstr=pot,
                         duplicatestyle=duplicatestyle)
@@ -318,7 +413,7 @@ def convertprop(inputfile, outputfile, templatefile, personality="java",
         outputstore = convertor.mergestore(templatestore, inputstore)
     if outputstore.isempty():
         return 0
-    outputfile.write(str(outputstore))
+    outputstore.serialize(outputfile)
     return 1
 
 
@@ -337,17 +432,18 @@ def main(argv=None):
     parser = convert.ConvertOptionParser(formats, usetemplates=True,
                                          usepots=True,
                                          description=__doc__)
-    parser.add_option("", "--personality", dest="personality",
-            default=properties.default_dialect,
-            type="choice",
-            choices=properties.dialects.keys(),
-            help="override the input file format: %s (for .properties files, default: %s)" %
-                 (", ".join(properties.dialects.iterkeys()),
-                  properties.default_dialect),
-            metavar="TYPE")
-    parser.add_option("", "--encoding", dest="encoding", default=None,
-            help="override the encoding set by the personality",
-            metavar="ENCODING")
+    parser.add_option(
+        "", "--personality", dest="personality",
+        default=properties.default_dialect,
+        type="choice",
+        choices=list(properties.dialects.keys()),
+        help="override the input file format: %s (for .properties files, default: %s)" % (
+            ", ".join(properties.dialects.keys()), properties.default_dialect),
+        metavar="TYPE")
+    parser.add_option(
+        "", "--encoding", dest="encoding", default=None,
+        help="override the encoding set by the personality",
+        metavar="ENCODING")
     parser.add_duplicates_option()
     parser.passthrough.append("pot")
     parser.passthrough.append("personality")
