@@ -1,13 +1,14 @@
+from io import BytesIO
+
 from translate.convert import json2po, test_convert
-from translate.misc import wStringIO
 from translate.storage import jsonl10n
 
 
-class TestJson2PO(object):
+class TestJson2PO:
 
     def json2po(self, jsonsource, template=None, filter=None):
         """helper that converts json source to po source without requiring files"""
-        inputfile = wStringIO.StringIO(jsonsource)
+        inputfile = BytesIO(jsonsource.encode())
         inputjson = jsonl10n.JsonFile(inputfile, filter=filter)
         convertor = json2po.json2po()
         outputpo = convertor.convert_store(inputjson)
@@ -77,9 +78,9 @@ class TestJson2POCommand(test_convert.TestConvertCommand, TestJson2PO):
     convertmodule = json2po
     defaultoptions = {"progress": "none"}
 
-    def test_help(self):
+    def test_help(self, capsys):
         """tests getting help"""
-        options = test_convert.TestConvertCommand.test_help(self)
+        options = test_convert.TestConvertCommand.test_help(self, capsys)
         options = self.help_check(options, "-P, --pot")
         options = self.help_check(options, "--duplicates")
         options = self.help_check(options, "-t TEMPLATE, --template=TEMPLATE")
