@@ -1,30 +1,29 @@
-# -*- coding: utf-8 -*-
+from io import BytesIO
 
 from translate.convert import php2po, test_convert
-from translate.misc import wStringIO
-from translate.storage import php, po
+from translate.storage import po
 
 
-class TestPhp2PO(object):
+class TestPhp2PO:
 
     def php2po(self, phpsource, phptemplate=None):
         """helper that converts .php source to po source without requiring files"""
-        inputfile = wStringIO.StringIO(phpsource)
-        output_file = wStringIO.StringIO()
+        inputfile = BytesIO(phpsource.encode())
+        output_file = BytesIO()
         templatefile = None
         if phptemplate:
-            templatefile = wStringIO.StringIO(phptemplate)
+            templatefile = BytesIO(phptemplate.encode())
         convertor = php2po.php2po(inputfile, output_file, templatefile)
         convertor.run()
         return convertor.target_store
 
     def convertphp(self, phpsource, template=None, expected=1):
         """call run_converter, return the outputfile"""
-        inputfile = wStringIO.StringIO(phpsource)
-        outputfile = wStringIO.StringIO()
+        inputfile = BytesIO(phpsource.encode())
+        outputfile = BytesIO()
         templatefile = None
         if template:
-            templatefile = wStringIO.StringIO(template)
+            templatefile = BytesIO(template.encode())
         assert php2po.run_converter(inputfile, outputfile, templatefile) == expected
         return outputfile.getvalue()
 
@@ -53,7 +52,7 @@ class TestPhp2PO(object):
         """checks that the convertphp function is working"""
         phpsource = """$_LANG['simple'] = 'entry';"""
         posource = self.convertphp(phpsource)
-        pofile = po.pofile(wStringIO.StringIO(posource))
+        pofile = po.pofile(BytesIO(posource))
         pounit = self.singleelement(pofile)
         assert pounit.source == "entry"
         assert pounit.target == ""
@@ -63,7 +62,7 @@ class TestPhp2PO(object):
         phpsource = """$_LANG['simple'] = 'entry';"""
         phptemplate = '''$_LANG['simple'] = 'source';'''
         posource = self.convertphp(phpsource, phptemplate)
-        pofile = po.pofile(wStringIO.StringIO(posource))
+        pofile = po.pofile(BytesIO(posource))
         pounit = self.singleelement(pofile)
         assert pounit.source == "source"
         assert pounit.target == "entry"
@@ -73,7 +72,7 @@ class TestPhp2PO(object):
         phpsource = """$_LANG['simple'] = 'entry';"""
         phptemplate = '''$_LANG['missing'] = 'source';'''
         posource = self.convertphp(phpsource, phptemplate)
-        pofile = po.pofile(wStringIO.StringIO(posource))
+        pofile = po.pofile(BytesIO(posource))
         pounit = self.singleelement(pofile)
         assert pounit.source == "source"
         assert pounit.target == ""
@@ -83,18 +82,18 @@ class TestPhp2PO(object):
         phpsource = ''
         phptemplate = ''
         posource = self.convertphp(phpsource, phptemplate, 0)
-        pofile = po.pofile(wStringIO.StringIO(posource))
+        pofile = po.pofile(BytesIO(posource))
         assert len(pofile.units) == 0
 
     def test_unicode(self):
         """checks that unicode entries convert properly"""
-        unistring = u'Norsk bokm\u00E5l'
+        unistring = 'Norsk bokm\u00E5l'
         phpsource = """$lang['nb'] = '%s';""" % unistring
         pofile = self.php2po(phpsource)
         pounit = self.singleelement(pofile)
         print(repr(pofile.units[0].target))
         print(repr(pounit.source))
-        assert pounit.source == u'Norsk bokm\u00E5l'
+        assert pounit.source == 'Norsk bokm\u00E5l'
 
     def test_multiline(self):
         """checks that multiline enties can be parsed"""
