@@ -1,12 +1,10 @@
-# -*- coding: utf-8 -*-
 
-from __future__ import unicode_literals
+from io import BytesIO
 
 from translate.convert import po2txt, test_convert
-from translate.misc import wStringIO
 
 
-class TestPO2Txt(object):
+class TestPO2Txt:
 
     ConverterClass = po2txt.po2txt
 
@@ -14,11 +12,11 @@ class TestPO2Txt(object):
                  output_threshold=None, encoding='utf-8', wrap=None,
                  success_expected=True):
         """Helper that converts to target format without using files."""
-        input_file = wStringIO.StringIO(input_string)
-        output_file = wStringIO.StringIO()
+        input_file = BytesIO(input_string.encode())
+        output_file = BytesIO()
         template_file = None
         if template_string:
-            template_file = wStringIO.StringIO(template_string)
+            template_file = BytesIO(template_string.encode())
         expected_result = 1 if success_expected else 0
         converter = self.ConverterClass(input_file, output_file, template_file,
                                         include_fuzzy, output_threshold,
