@@ -1,4 +1,3 @@
-# -*- coding: utf-8 -*-
 #
 # Copyright 2016 Zuza Software Foundation
 #
@@ -22,14 +21,15 @@
 .. seealso:: https://en.wikipedia.org/wiki/Danish_language
 """
 
+
 from translate.lang import common
 
 
 class da(common.Common):
     """This class represents Danish."""
 
-    validaccel = (u"abcdefghijklmnopqrstuvwxyz"
-                  u"ABCDEFGHIJKLMNOPQRSTUVWXYZ"
-                  u"1234567890"
-                  u"æøåé"
-                  u"ÆØÅÉ")
+    validaccel = ("abcdefghijklmnopqrstuvwxyz"
+                  "ABCDEFGHIJKLMNOPQRSTUVWXYZ"
+                  "1234567890"
+                  "æøåé"
+                  "ÆØÅÉ")
