@@ -1,4 +1,3 @@
-# -*- coding: utf-8 -*-
 #
 # Copyright 2007-2011 Zuza Software Foundation
 #
@@ -19,13 +18,12 @@
 
 """This module stores information and functionality that relates to plurals."""
 
-from __future__ import unicode_literals
 
 import gettext
 import locale
 import os
 import re
-import six
+
 
 try:
     import pycountry
@@ -165,6 +163,7 @@ languages = {
     'sah': ('Yakut', 1, '0'),
     'sat': ('Santali', 2, '(n != 1)'),
     'sco': ('Scots', 2, '(n != 1)'),
+    'scn': ('Sicilian', 2, '(n != 1)'),
     'sd': ('Sindhi', 2, '(n != 1)'),
     'se': ('Northern Sami', 2, '(n != 1)'),
     'si': ('Sinhala; Sinhalese', 2, '(n != 1)'),
@@ -336,6 +335,7 @@ plural_tags = {
     'af': ['one', 'other'],
     'ak': ['one', 'other'],
     'am': ['one', 'other'],
+    'an': ['one', 'other'],
     'ar': ['zero', 'one', 'two', 'few', 'many', 'other'],
     'ars': ['zero', 'one', 'two', 'few', 'many', 'other'],
     'as': ['one', 'other'],
@@ -346,7 +346,7 @@ plural_tags = {
     'bem': ['one', 'other'],
     'bez': ['one', 'other'],
     'bg': ['one', 'other'],
-    'bh': ['one', 'other'],
+    'bho': ['one', 'other'],
     'bm': ['other'],
     'bn': ['one', 'other'],
     'bo': ['other'],
@@ -467,6 +467,7 @@ plural_tags = {
     'om': ['one', 'other'],
     'or': ['one', 'other'],
     'os': ['one', 'other'],
+    'osa': ['other'],
     'pa': ['one', 'other'],
     'pap': ['one', 'other'],
     'pl': ['one', 'few', 'many'],
@@ -507,6 +508,7 @@ plural_tags = {
     'ss': ['one', 'other'],
     'ssy': ['one', 'other'],
     'st': ['one', 'other'],
+    'su': ['other'],
     'sv': ['one', 'other'],
     'sw': ['one', 'other'],
     'syr': ['one', 'other'],
@@ -540,7 +542,7 @@ plural_tags = {
     'yo': ['other'],
     'yue': ['other'],
     'zh': ['other'],
-    'zu': ['one', 'other']
+    'zu': ['one', 'other'],
 }
 
 
@@ -600,7 +602,7 @@ def get_country_iso_name(country_code):
             return country.common_name
         return country.name
     except (KeyError, AttributeError):
-        return u""
+        return ""
 
 
 def get_language_iso_name(language_code):
@@ -614,7 +616,7 @@ def get_language_iso_name(language_code):
             return language.common_name
         return language.name
     except (KeyError, AttributeError):
-        return u""
+        return ""
 
 
 def get_language_iso_fullname(language_code):
@@ -627,15 +629,15 @@ def get_language_iso_fullname(language_code):
         language_code = language_code.replace("_", "-").replace("@", "-")
         language_code = "-".join(language_code.split("-")[:2])
         if "-" not in language_code:
-            return u""
+            return ""
         language_code, country_code = language_code.split("-")
         language_name = get_language_iso_name(language_code)
         if not language_name:
-            return u""
+            return ""
         country_name = get_country_iso_name(country_code)
         if not country_name:
-            return u""
-        return u"%s (%s)" % (language_name, country_name)
+            return ""
+        return "%s (%s)" % (language_name, country_name)
     return get_language_iso_name(language_code)
 
 
@@ -658,7 +660,7 @@ def tr_lang(langcode=None):
             language, country = match.groups()
             if country != "macrolanguage":
                 return (
-                    u"%s (%s)"
+                    "%s (%s)"
                     % (_fix_language_name(langfunc(language)),
                        countryfunc(country)))
         return _fix_language_name(langfunc(name))
@@ -697,7 +699,7 @@ def gettext_domain(langcode, domain, localedir=None):
         # On Windows the default locale is not used for some reason
         kwargs['languages'] = [locale.getdefaultlocale()[0]]
     t = gettext.translation(**kwargs)
-    return t.ugettext if six.PY2 else t.gettext
+    return t.gettext
 
 
 def gettext_lang(langcode=None):
@@ -747,7 +749,7 @@ def forceunicode(string):
         encoding = getattr(string, "encoding", "utf-8")
         string = string.decode(encoding)
     elif isinstance(string, StringElem):
-        string = six.text_type(string)
+        string = str(string)
     return string
 
 
@@ -765,7 +767,7 @@ def normalize_code(code):
 __normalised_languages = set(normalize_code(key) for key in languages.keys())
 
 
-def simplify_to_common(language_code, languages=languages):
+def simplify_to_common(language_code):
     """Simplify language code to the most commonly used form for the language,
     stripping country information for languages that tend not to be localized
     differently for different countries
