@@ -1,7 +1,5 @@
-#!/usr/bin/env python2
-# -*- coding: utf-8 -*-
 #
-# Copyright 2008 Zuza Software Foundation
+# Copyright 2008, 2016 Zuza Software Foundation
 #
 # This file is part of translate.
 #
@@ -23,10 +21,13 @@
 .. seealso:: http://en.wikipedia.org/wiki/Kannada_language
 """
 
+
 from translate.lang import common
 
 
 class kn(common.Common):
     """This class represents Kannada."""
 
-    ignoretests = ["startcaps", "simplecaps"]
+    ignoretests = {
+        'all': ["simplecaps", "startcaps"],
+    }
