@@ -1,4 +1,3 @@
-# -*- coding: utf-8 -*-
 #
 # Copyright 2013, 2016 Zuza Software Foundation
 #
@@ -22,7 +21,6 @@
 .. seealso:: http://en.wikipedia.org/wiki/Burmese_language
 """
 
-from __future__ import unicode_literals
 
 from translate.lang import common
 
