@@ -1,4 +1,3 @@
-# -*- coding: utf-8 -*-
 #
 # Copyright 2010 Zuza Software Foundation
 #
@@ -20,12 +19,9 @@
 """Module to guess the language ISO code based on the 'Language-Team' entry in
 the header of a Gettext PO file.
 """
-from __future__ import print_function, unicode_literals
 
 import re
 
-import six
-
 
 __all__ = ('LANG_TEAM_CONTACT_SNIPPETS', 'guess_language')
 
@@ -37,7 +33,7 @@ LANG_TEAM_REGEX = (
     ("ubuntu-l10n", "ubuntu-l10n-([a-z_A-Z]+)@lists.ubuntu.com", None),
     ("translate-discuss",
      "translate-discuss-([a-z_A-Z]+)@lists.sourceforge.net", None),
-    ("kde-i18n", "kde-i18n-([a-z_A-Z]+)@(?:lists\.|mail\.|)kde.org", ["doc"]),
+    ("kde-i18n", r"kde-i18n-([a-z_A-Z]+)@(?:lists\.|mail\.|)kde.org", ["doc"]),
     ("kde-l10n", "kde-l10n-([a-z_A-Z]+)@kde.org", None),
     ("fedoraproject", "trans-([a-z_A-Z]+)@lists.fedoraproject.org", None),
     ("gnome.org", "gnome-([a-z_A-Z]+)-list@gnome.org", ["latin"]),
@@ -187,6 +183,7 @@ LANG_TEAM_CONTACT_SNIPPETS = {
            "debian-l10n-romanian@lists.debian.org", ),
     "ru": ("pgsql-rus@yahoogroups.com", "debian-l10n-russian@lists.debian.org",
            "gnupg-ru@gnupg.org", ),
+    "scn": ("l10n@cademiasiciliana.org", ),
     "sk": ("sk-i18n@lists.linux.sk", "kde-sk@linux.sk", ),
     "sl": ("gnome-si@googlegroups.com", ),
     "sq": ("gnome-albanian-perkthyesit@lists.sourceforge.net",
@@ -314,6 +311,7 @@ LANG_TEAM_LANGUAGE_SNIPPETS = {
     "rm": ("Rhaeto-Romance", ),
     "ro": ("Romania", "Romanian", "Română", ),
     "ru": ("Russian", ),
+    "scn": ("Sicilian", "Sicilianu", "Siciliano", ),
     "si": ("Sinhala", "Sinhalese", ),
     "sk": ("Slovak", ),
     "sl": ("Slovene", "Slovenian", ),
@@ -404,7 +402,7 @@ def _snippet_guesser(snippets_dict, string, filter_=_nofilter):
     before examination
     """
     string = filter_(string)
-    for possible_lang, snippets in six.iteritems(snippets_dict):
+    for possible_lang, snippets in snippets_dict.items():
         for snippet in snippets:
             if filter_(snippet) in string:
                 return possible_lang
