@@ -1,4 +1,3 @@
-# -*- coding: latin-1 -*-
 """selector - WSGI delegation based on URL path and method.
 
 (See the docstring of selector.Selector.)
@@ -25,7 +24,6 @@ Luke Arno can be found at http://lukearno.com/
 """
 
 import re
-import six
 from itertools import starmap
 from wsgiref.util import shift_path_info
 
@@ -60,7 +58,7 @@ def not_found(environ, start_response):
             "The server has not found anything matching the Request-URI."]
 
 
-class Selector(object):
+class Selector:
     """WSGI middleware for URL paths and HTTP method based delegation.
     
     See http://lukearno.com/projects/selector/
@@ -125,7 +123,7 @@ class Selector(object):
         
         Prefix will override self.prefix for this mapping.
         """
-        # Thanks to Sbastien Pierre 
+        # Thanks to Sébastien Pierre
         # for suggesting that this accept keyword args.
         if method_dict is None:
             method_dict = {}
@@ -145,7 +143,7 @@ class Selector(object):
         app, svars, methods, matched = \
             self.select(environ['PATH_INFO'], environ['REQUEST_METHOD'])
         unnamed, named = [], {}
-        for k, v in six.iteritems(svars):
+        for k, v in svars.items():
             if k.startswith('__pos'):
                 k = k[5:]
             named[k] = v
@@ -215,7 +213,7 @@ class Selector(object):
         ``@prefix`` and ``@parser`` directives take effect 
         until the end of the file or until changed.
         """
-        if isinstance(the_file, six.string_types):
+        if isinstance(the_file, str):
             the_file = open(the_file)
         oldprefix = self.prefix
         if prefix is not None:
@@ -281,8 +279,8 @@ class Selector(object):
         return path, methods
 
 
-class SimpleParser(object):
-    """Callable to turn path expressions into regexes with named groups.
+class SimpleParser:
+    r"""Callable to turn path expressions into regexes with named groups.
     
     For instance ``"/hello/{name}"`` becomes ``r"^\/hello\/(?P<name>[^\^.]+)$"``
 
@@ -406,7 +404,7 @@ class SimpleParser(object):
             return self.lastly(self.parse(url_pattern))
 
 
-class EnvironDispatcher(object):
+class EnvironDispatcher:
     """Dispatch based on list of rules."""
 
     def __init__(self, rules):
@@ -423,7 +421,7 @@ class EnvironDispatcher(object):
                 return app(environ, start_response)
 
 
-class MiddlewareComposer(object):
+class MiddlewareComposer:
     """Compose middleware based on list of rules."""
 
     def __init__(self, app, rules):
@@ -459,7 +457,7 @@ def expose(obj):
     return obj
 
 
-class Naked(object):
+class Naked:
     """Naked object style dispatch base class."""
 
     _not_found = staticmethod(not_found)
@@ -486,7 +484,7 @@ class Naked(object):
             return self._not_found(environ, start_response)
     
 
-class ByMethod(object):
+class ByMethod:
     """Base class for dispatching to method named by ``REQUEST_METHOD``."""
 
     _method_not_allowed = staticmethod(method_not_allowed)
@@ -523,7 +521,7 @@ def opliant(meth):
     ``wsgiorg.routing_args``
     ::
 
-        class App(object):
+        class App:
             @opliant
             def __call__(self, environ, start_response, arg1, arg2, foo='bar'):
                 ...
