@@ -18,6 +18,7 @@
 
 """Base classes for storage interfaces."""
 
+from collections import OrderedDict
 import codecs
 import logging
 import pickle
@@ -132,7 +133,7 @@ class TranslationUnit:
         :return: Returns *True* if the supplied :class:`TranslationUnit`
                  equals this unit.
         """
-        return self.source == other.source and self.target == other.target
+        return self.source == other.source and self.target == other.target and self.getid() == other.getid()
 
     def __ne__(self, other):
         return not self.__eq__(other)
@@ -455,6 +456,7 @@ class TranslationUnit:
         newunit = cls(unit.source)
         newunit.target = unit.target
         newunit.markfuzzy(unit.isfuzzy())
+        newunit.setid(unit.getid())
         locations = unit.getlocations()
         if locations:
             newunit.addlocations(locations)
@@ -521,6 +523,9 @@ class TranslationStore:
         if unitclass:
             self.UnitClass = unitclass
         self._encoding = encoding
+        self.locationindex = {}
+        self.sourceindex = {}
+        self.id_index = {}
 
     @property
     def encoding(self):
@@ -582,6 +587,18 @@ class TranslationStore:
         unit._store = self
         self.units.append(unit)
 
+    def removeunit(self, unit):
+        """Remove the given unit to the object's list of units.
+
+        This method should always be used rather than trying to modify the
+        list manually.
+
+        :type unit: :class:`TranslationUnit`
+        :param unit: The unit that will be added.
+        """
+        self.units.remove(unit)
+        self.remove_unit_from_index(unit)
+
     def addsourceunit(self, source):
         """Add and returns a new unit with the given source string.
 
@@ -601,13 +618,9 @@ class TranslationStore:
 
         :rtype: :class:`TranslationUnit` or None
         """
-        if len(getattr(self, "sourceindex", [])):
-            if source in self.sourceindex:
-                return self.sourceindex[source][0]
-        else:
-            for unit in self.units:
-                if unit.source == source:
-                    return unit
+        self.require_index()
+        if source in self.sourceindex:
+            return self.sourceindex[source][0]
         return None
 
     def findunits(self, source):
@@ -615,17 +628,9 @@ class TranslationStore:
 
         :rtype: :class:`TranslationUnit` or None
         """
-        if len(getattr(self, "sourceindex", [])):
-            if source in self.sourceindex:
-                return self.sourceindex[source]
-        else:
-            #FIXME: maybe we should generate index here instead since
-            #we'll scan all units anyway
-            result = []
-            for unit in self.units:
-                if unit.source == source:
-                    result.append(unit)
-            return result
+        self.require_index()
+        if source in self.sourceindex:
+            return self.sourceindex[source]
         return None
 
     def translate(self, source):
@@ -642,20 +647,17 @@ class TranslationStore:
     def remove_unit_from_index(self, unit):
         """Remove a unit from source and locaton indexes"""
 
-        def remove_unit(source):
+        def remove_source(source):
             if source in self.sourceindex:
-                try:
-                    self.sourceindex[source].remove(unit)
-                    if len(self.sourceindex[source]) == 0:
-                        del(self.sourceindex[source])
-                except ValueError:
-                    pass
+                self.sourceindex[source].remove(unit)
+                if len(self.sourceindex[source]) == 0:
+                    del(self.sourceindex[source])
 
         if unit.hasplural():
             for source in unit.source.strings:
-                remove_unit(source)
+                remove_source(source)
         else:
-            remove_unit(unit.source)
+            remove_source(unit.source)
 
         for location in unit.getlocations():
             if (location in self.locationindex and
@@ -700,7 +702,7 @@ class TranslationStore:
 
     def require_index(self):
         """make sure source index exists"""
-        if not hasattr(self, "id_index"):
+        if not self.id_index:
             self.makeindex()
 
     def getids(self, filename=None):
@@ -887,3 +889,109 @@ class TranslationStore:
         :rtype: string
         """
         return "id"
+
+
+class UnitId:
+    KEY_SEPARATOR = "."
+    INDEX_SEPARATOR = ""
+
+    def __init__(self, parts):
+        self.parts = parts
+
+    def __str__(self):
+        def fmt(element, key):
+            if element == 'key':
+                return '{}{}'.format(self.KEY_SEPARATOR, key)
+            elif element == 'index':
+                return '{}[{}]'.format(self.INDEX_SEPARATOR, key)
+            else:
+                raise ValueError('Unsupported element: {}'.format(element))
+        return ''.join([fmt(*part) for part in self.parts])
+
+    def __add__(self, other):
+        if not isinstance(other, list):
+            raise ValueError('Not supported type for add: {}'.format(type(other)))
+        return self.__class__(self.parts + other)
+
+    @classmethod
+    def from_string(cls, text):
+        result = []
+        # Strip possible leading separator
+        if text.startswith(cls.KEY_SEPARATOR):
+            text = text[len(cls.KEY_SEPARATOR):]
+        for item in text.split(cls.KEY_SEPARATOR):
+            if '[' in item and item[-1] == ']':
+                item, pos = item[:-1].split('[')
+                if cls.INDEX_SEPARATOR and item:
+                    result.append(("key", item))
+                result.append(("index", int(pos)))
+            else:
+                result.append(("key", item))
+        return cls(result)
+
+
+class DictUnit(TranslationUnit):
+    IdClass = UnitId
+    DefaultDict = OrderedDict
+
+    def __init__(self, source=None):
+        super().__init__(source)
+        self._unitid = None
+
+    def storevalue(self, output, value, override_key=None, unset=False):
+        target = output
+        if self._unitid is None:
+            self._unitid = self.IdClass.from_string(self._id)
+        parts = self._unitid.parts
+        for pos, part in enumerate(parts[:-1]):
+            element, key = part
+            default = [] if parts[pos + 1][0] == 'index' else self.DefaultDict()
+            if element == 'index':
+                if len(target) <= key and not unset:
+                    target.append(default)
+            elif element == 'key':
+                if key not in target or isinstance(target[key], str):
+                    target[key] = default
+            else:
+                raise ValueError('Unsupported element: {}'.format(element))
+            target = target[key]
+        if override_key:
+            element, key = 'key', override_key
+        else:
+            element, key = parts[-1]
+        if element == 'key':
+            if unset:
+                del target[key]
+            else:
+                target[key] = value
+        elif element == 'index':
+            if len(target) <= key:
+                if not unset:
+                    target.append(value)
+            else:
+                if unset:
+                    del target[key]
+                else:
+                    target[key] = value
+        else:
+            raise ValueError('Unsupported element: {}'.format(element))
+
+    def storevalues(self, output):
+        self.storevalue(output, self.value)
+
+    def getvalue(self):
+        """Returns dictionary for serialization."""
+        result = {}
+        self.storevalues(result)
+        return result
+
+    def set_unitid(self, unitid):
+        self.setid(str(unitid))
+        self._unitid = unitid
+
+
+class DictStore(TranslationStore):
+
+    def serialize_units(self, output):
+        for unit in self.unit_iter():
+            unit.storevalues(output)
