@@ -1,5 +1,3 @@
-#!/usr/bin/env python2
-# -*- coding: utf-8 -*-
 #
 # Copyright 2002-2007 Zuza Software Foundation
 #
@@ -41,7 +39,6 @@ from ctypes import (CFUNCTYPE, POINTER, Structure, c_char_p, c_int, c_long,
 from translate.lang import data
 from translate.misc.multistring import multistring
 from translate.storage import base, pocommon, pypo
-from translate.storage.pocommon import encodingToUse
 
 
 logger = logging.getLogger(__name__)
@@ -54,14 +51,40 @@ STRING = c_char_p
 
 # Structures
 class po_message(Structure):
-    _fields_ = []
+    pass
+
+
+class po_file(Structure):
+    pass
+
+
+class po_filepos(Structure):
+    pass
+
+
+class po_iterator(Structure):
+    pass
+
+
+po_message_t = POINTER(po_message)
+"""A po_message_t represents a message in a PO file."""
+
+po_file_t = POINTER(po_file)
+"""A po_file_t represents a PO file."""
+
+po_filepos_t = POINTER(po_filepos)
+"""A po_filepos_t represents the position in a PO file."""
+
+po_iterator_t = POINTER(po_iterator)
+"""A po_iterator_t represents an iterator through a PO file."""
+
 
 # Function prototypes
-xerror_prototype = CFUNCTYPE(None, c_int, POINTER(po_message), STRING, c_uint,
+xerror_prototype = CFUNCTYPE(None, c_int, po_message_t, STRING, c_uint,
                              c_uint, c_int, STRING)
-xerror2_prototype = CFUNCTYPE(None, c_int, POINTER(po_message), STRING,
+xerror2_prototype = CFUNCTYPE(None, c_int, po_message_t, STRING,
                               c_uint, c_uint, c_int, STRING,
-                              POINTER(po_message), STRING, c_uint, c_uint,
+                              po_message_t, STRING, c_uint, c_uint,
                               c_int, STRING)
 
 
@@ -83,8 +106,9 @@ class po_error_handler(Structure):
 # Callback functions for po_xerror_handler
 def xerror_cb(severity, message, filename, lineno, column, multiline_p,
               message_text):
-    logger.error("xerror_cb" + severity + message +
-                 filename + lineno + column + multiline_p + message_text)
+    logger.error("xerror_cb %s %s %s %s %s %s %s" % (
+        severity, message, filename, lineno, column, multiline_p, message_text
+    ))
     if severity >= 1:
         raise ValueError(message_text)
 
@@ -92,52 +116,97 @@ def xerror_cb(severity, message, filename, lineno, column, multiline_p,
 def xerror2_cb(severity, message1, filename1, lineno1, column1, multiline_p1,
                message_text1, message2, filename2, lineno2, column2,
                multiline_p2, message_text2):
-    logger.error("xerror2_cb" + severity +
-                 message1 +
-                 filename1 + lineno1 + column1 + multiline_p1 + message_text1,
-                 filename2 + lineno2 + column2 + multiline_p2 + message_text2)
+    logger.error("xerror2_cb %s %s %s %s %s %s %s %s %s %s %s %s" % (
+        severity, message1,
+        filename1, lineno1, column1, multiline_p1, message_text1,
+        filename2, lineno2, column2, multiline_p2, message_text2
+    ))
     if severity >= 1:
         raise ValueError(message_text1)
 
 
 # Setup return and parameter types
+# See also http://git.savannah.gnu.org/cgit/gettext.git/tree/gettext-tools/libgettextpo/gettext-po.in.h
 def setup_call_types(gpo):
     # File access
+    gpo.po_file_create.restype = po_file_t
     gpo.po_file_read_v3.argtypes = [STRING, POINTER(po_xerror_handler)]
-    gpo.po_file_write_v2.argtypes = [c_int, STRING, POINTER(po_xerror_handler)]
-    gpo.po_file_write_v2.retype = c_int
+    gpo.po_file_read_v3.restype = po_file_t
+    gpo.po_file_write_v2.argtypes = [po_file_t, STRING, POINTER(po_xerror_handler)]
+    gpo.po_file_write_v2.restype = po_file_t
+    gpo.po_file_free.argtypes = [po_file_t]
 
     # Header
+    gpo.po_file_domain_header.argtypes = [po_file_t, STRING]
     gpo.po_file_domain_header.restype = STRING
-    gpo.po_header_field.restype = STRING
     gpo.po_header_field.argtypes = [STRING, STRING]
+    gpo.po_header_field.restype = STRING
+    gpo.po_header_set_field.argtypes = [STRING, STRING, STRING]
+    gpo.po_header_set_field.restype = STRING
 
     # Locations (filepos)
+    gpo.po_filepos_file.argtypes = [po_filepos_t]
     gpo.po_filepos_file.restype = STRING
-    gpo.po_message_filepos.restype = c_int
-    gpo.po_message_filepos.argtypes = [c_int, c_int]
-    gpo.po_message_add_filepos.argtypes = [c_int, STRING, c_size_t]
+    gpo.po_filepos_start_line.argtypes = [po_filepos_t]
+    gpo.po_filepos_start_line.restype = c_int  # not strictly true casting
+    gpo.po_message_filepos.argtypes = [po_message_t, c_int]
+    gpo.po_message_filepos.restype = po_filepos_t
+    gpo.po_message_add_filepos.argtypes = [po_message_t, STRING, c_size_t]
+    gpo.po_message_remove_filepos.argtypes = [po_message_t, c_size_t]
+
+    # Iterators
+    gpo.po_message_iterator.argtypes = [po_file_t, STRING]
+    gpo.po_message_iterator.restype = po_iterator_t
+    gpo.po_message_iterator_free.argtypes = [po_iterator_t]
+    gpo.po_next_message.argtypes = [po_iterator_t]
+    gpo.po_next_message.restype = po_message_t
+    gpo.po_message_insert.argtypes = [po_iterator_t, po_message_t]
 
     # Message (get methods)
-    gpo.po_message_comments.restype = STRING
-    gpo.po_message_extracted_comments.restype = STRING
-    gpo.po_message_prev_msgctxt.restype = STRING
-    gpo.po_message_prev_msgid.restype = STRING
-    gpo.po_message_prev_msgid_plural.restype = STRING
-    gpo.po_message_is_format.restype = c_int
-    gpo.po_message_is_format.argtypes = [c_int, STRING]
-    gpo.po_message_set_format.argtypes = [c_int, STRING, c_int]
+    gpo.po_message_create.restype = po_message_t
+    gpo.po_message_msgctxt.argtypes = [po_message_t]
     gpo.po_message_msgctxt.restype = STRING
+    gpo.po_message_comments.argtypes = [po_message_t]
+    gpo.po_message_comments.restype = STRING
+    gpo.po_message_extracted_comments.argtypes = [po_message_t]
+    gpo.po_message_extracted_comments.restype = STRING
+    gpo.po_message_prev_msgctxt.argtypes = [po_message_t]
+    gpo.po_message_prev_msgctxt.restype = STRING
+    gpo.po_message_prev_msgid.argtypes = [po_message_t]
+    gpo.po_message_prev_msgid.restype = STRING
+    gpo.po_message_prev_msgid_plural.argtypes = [po_message_t]
+    gpo.po_message_prev_msgid_plural.restype = STRING
+    gpo.po_message_is_obsolete.argtypes = [po_message_t]
+    gpo.po_message_is_obsolete.restype = c_int
+    gpo.po_message_is_fuzzy.argtypes = [po_message_t]
+    gpo.po_message_is_fuzzy.restype = c_int
+    gpo.po_message_is_format.argtypes = [po_message_t, STRING]
+    gpo.po_message_is_format.restype = c_int
+    gpo.po_message_msgctxt.restype = STRING
+    gpo.po_message_msgid.argtypes = [po_message_t]
     gpo.po_message_msgid.restype = STRING
+    gpo.po_message_msgid_plural.argtypes = [po_message_t]
     gpo.po_message_msgid_plural.restype = STRING
+    gpo.po_message_msgstr.argtypes = [po_message_t]
     gpo.po_message_msgstr.restype = STRING
+    gpo.po_message_msgstr_plural.argtypes = [po_message_t, c_int]
     gpo.po_message_msgstr_plural.restype = STRING
 
     # Message (set methods)
-    gpo.po_message_set_comments.argtypes = [c_int, STRING]
-    gpo.po_message_set_extracted_comments.argtypes = [c_int, STRING]
-    gpo.po_message_set_fuzzy.argtypes = [c_int, c_int]
-    gpo.po_message_set_msgctxt.argtypes = [c_int, STRING]
+    gpo.po_message_set_comments.argtypes = [po_message_t, STRING]
+    gpo.po_message_set_extracted_comments.argtypes = [po_message_t, STRING]
+    gpo.po_message_set_prev_msgctxt.argtypes = [po_message_t, STRING]
+    gpo.po_message_set_prev_msgid.argtypes = [po_message_t, STRING]
+    gpo.po_message_set_prev_msgid_plural.argtypes = [po_message_t, STRING]
+    gpo.po_message_set_obsolete.argtypes = [po_message_t, c_int]
+    gpo.po_message_set_fuzzy.argtypes = [po_message_t, c_int]
+    gpo.po_message_set_format.argtypes = [po_message_t, STRING, c_int]
+    gpo.po_message_set_msgctxt.argtypes = [po_message_t, STRING]
+    gpo.po_message_set_msgid.argtypes = [po_message_t, STRING]
+    gpo.po_message_set_msgstr.argtypes = [po_message_t, STRING]
+    gpo.po_message_set_msgstr_plural.argtypes = [po_message_t, c_int, STRING]
+    gpo.po_message_set_range.argtypes = [po_message_t, c_int, c_int]
+
 
 # Load libgettextpo
 gpo = None
@@ -188,9 +257,9 @@ def unquotefrompo(postr):
 def get_libgettextpo_version():
     """Returns the libgettextpo version
 
-       :rtype: three-value tuple
-       :return: libgettextpo version in the following format::
-           (major version, minor version, subminor version)
+    :rtype: three-value tuple
+    :return: libgettextpo version in the following format::
+        (major version, minor version, subminor version)
     """
     libversion = c_long.in_dll(gpo, 'libgettextpo_version')
     major = libversion.value >> 16
@@ -199,6 +268,18 @@ def get_libgettextpo_version():
     return major, minor, subminor
 
 
+def gpo_encode(value):
+    return value.encode('utf-8') if isinstance(value, str) else value
+
+
+def gpo_decode(value):
+    if isinstance(value, str):
+        return value
+    elif isinstance(value, bytes):
+        return value.decode('utf-8')
+    return value
+
+
 class pounit(pocommon.pounit):
 
     #: fixed encoding that is always used for cPO structure (self._gpo_message)
@@ -218,13 +299,24 @@ class pounit(pocommon.pounit):
                 features = ['msgctxt', 'msgid', 'msgid_plural']
                 features += ['prev_' + x for x in features]
                 features += ['comments', 'extracted_comments',
-                             'msgstr', 'msgstr_plural']
+                             'msgstr']
                 for feature in features:
                     text = getattr(gpo, 'po_message_' + feature)(gpo_message)
                     if text:
                         getattr(gpo, 'po_message_set_' + feature)(
                             gpo_message,
                             text.decode(encoding).encode(self.CPO_ENC))
+                # Also iterate through plural forms
+                nplural = 0
+                text = True
+                while text:
+                    text = gpo.po_message_msgstr_plural(gpo_message, nplural)
+                    if text:
+                        gpo.po_message_set_msgstr_plural(
+                            gpo_message,
+                            text.decode(encoding).encode(self.CPO_ENC),
+                            nplural)
+                    nplural += 1
             self._gpo_message = gpo_message
         self.infer_state()
 
@@ -237,7 +329,7 @@ class pounit(pocommon.pounit):
                 self.set_state_n(self.STATE[self.S_OBSOLETE][0])
         elif gpo.po_message_is_fuzzy(self._gpo_message):
             self.set_state_n(self.STATE[self.S_FUZZY][0])
-        elif self.gettarget():
+        elif self.target:
             self.set_state_n(self.STATE[self.S_TRANSLATED][0])
         else:
             self.set_state_n(self.STATE[self.S_UNTRANSLATED][0])
@@ -245,11 +337,11 @@ class pounit(pocommon.pounit):
     def setmsgid_plural(self, msgid_plural):
         if isinstance(msgid_plural, list):
             msgid_plural = "".join(msgid_plural)
-        gpo.po_message_set_msgid_plural(self._gpo_message, msgid_plural)
+        gpo.po_message_set_msgid_plural(self._gpo_message, gpo_encode(msgid_plural))
     msgid_plural = property(None, setmsgid_plural)
 
-    def getsource(self):
-
+    @property
+    def source(self):
         def remove_msgid_comments(text):
             if not text:
                 return text
@@ -258,36 +350,35 @@ class pounit(pocommon.pounit):
                 if remainder:
                     return remainder.group(1)
                 else:
-                    return u""
+                    return ""
             else:
                 return text
-        singular = remove_msgid_comments((gpo.po_message_msgid(self._gpo_message) or "").decode(self.CPO_ENC))
+        singular = remove_msgid_comments(gpo_decode(gpo.po_message_msgid(self._gpo_message)) or "")
         if singular:
             if self.hasplural():
-                multi = multistring(singular, self.CPO_ENC)
-                pluralform = (gpo.po_message_msgid_plural(self._gpo_message) or "").decode(self.CPO_ENC)
+                multi = multistring(singular)
+                pluralform = gpo_decode(gpo.po_message_msgid_plural(self._gpo_message)) or ""
                 multi.strings.append(pluralform)
                 return multi
             else:
                 return singular
         else:
-            return u""
+            return ""
 
-    def setsource(self, source):
+    @source.setter
+    def source(self, source):
         if isinstance(source, multistring):
             source = source.strings
-        if isinstance(source, unicode):
-            source = source.encode(self.CPO_ENC)
         if isinstance(source, list):
-            gpo.po_message_set_msgid(self._gpo_message, source[0].encode(self.CPO_ENC))
+            gpo.po_message_set_msgid(self._gpo_message, gpo_encode(source[0]))
             if len(source) > 1:
-                gpo.po_message_set_msgid_plural(self._gpo_message, source[1].encode(self.CPO_ENC))
+                gpo.po_message_set_msgid_plural(self._gpo_message, gpo_encode(source[1]))
         else:
-            gpo.po_message_set_msgid(self._gpo_message, source)
+            gpo.po_message_set_msgid(self._gpo_message, gpo_encode(source))
             gpo.po_message_set_msgid_plural(self._gpo_message, None)
-    source = property(getsource, setsource)
 
-    def gettarget(self):
+    @property
+    def target(self):
         if self.hasplural():
             plurals = []
             nplural = 0
@@ -297,19 +388,20 @@ class pounit(pocommon.pounit):
                 nplural += 1
                 plural = gpo.po_message_msgstr_plural(self._gpo_message, nplural)
             if plurals:
-                multi = multistring(plurals, encoding=self.CPO_ENC)
+                multi = multistring(plurals)
             else:
-                multi = multistring(u"")
+                multi = multistring("")
         else:
-            multi = (gpo.po_message_msgstr(self._gpo_message) or "").decode(self.CPO_ENC)
+            multi = gpo_decode(gpo.po_message_msgstr(self._gpo_message)) or ""
         return multi
 
-    def settarget(self, target):
+    @target.setter
+    def target(self, target):
         # for plural strings: convert 'target' into a list
         if self.hasplural():
             if isinstance(target, multistring):
                 target = target.strings
-            elif isinstance(target, basestring):
+            elif isinstance(target, str):
                 target = [target]
         # for non-plurals: check number of items in 'target'
         elif isinstance(target, (dict, list)):
@@ -331,29 +423,24 @@ class pounit(pocommon.pounit):
                 message = gpo.po_message_msgstr_plural(self._gpo_message, i)
         # add the items of a list
         if isinstance(target, list):
-            for i in range(len(target)):
-                targetstring = target[i]
-                if isinstance(targetstring, unicode):
-                    targetstring = targetstring.encode(self.CPO_ENC)
-                gpo.po_message_set_msgstr_plural(self._gpo_message, i, targetstring)
+            for i, targetstring in enumerate(target):
+                gpo.po_message_set_msgstr_plural(self._gpo_message, i, gpo_encode(targetstring))
         # add the values of a dict
         elif isinstance(target, dict):
-            for i, targetstring in enumerate(target.itervalues()):
-                gpo.po_message_set_msgstr_plural(self._gpo_message, i, targetstring)
+            for i, targetstring in enumerate(target.values()):
+                gpo.po_message_set_msgstr_plural(self._gpo_message, i, gpo_encode(targetstring))
         # add a single string
         else:
-            if isinstance(target, unicode):
-                target = target.encode(self.CPO_ENC)
             if target is None:
-                gpo.po_message_set_msgstr(self._gpo_message, "")
+                gpo.po_message_set_msgstr(self._gpo_message, gpo_encode(""))
             else:
-                gpo.po_message_set_msgstr(self._gpo_message, target)
-    target = property(gettarget, settarget)
+                gpo.po_message_set_msgstr(self._gpo_message, gpo_encode(target))
 
     def getid(self):
         """The unique identifier for this unit according to the conventions in
-        .mo files."""
-        id = (gpo.po_message_msgid(self._gpo_message) or "").decode(self.CPO_ENC)
+        .mo files.
+        """
+        id = gpo_decode(gpo.po_message_msgid(self._gpo_message)) or ""
         # Gettext does not consider the plural to determine duplicates, only
         # the msgid. For generation of .mo files, we might want to use this
         # code to generate the entry for the hash table, but for now, it is
@@ -363,13 +450,13 @@ class pounit(pocommon.pounit):
 #            id = '%s\0%s' % (id, plural)
         context = gpo.po_message_msgctxt(self._gpo_message)
         if context:
-            id = u"%s\04%s" % (context.decode(self.CPO_ENC), id)
+            id = "%s\04%s" % (gpo_decode(context), id)
         return id
 
     def getnotes(self, origin=None):
         if origin is None:
-            comments = gpo.po_message_comments(self._gpo_message) + \
-                       gpo.po_message_extracted_comments(self._gpo_message)
+            comments = (gpo.po_message_comments(self._gpo_message) +
+                        gpo.po_message_extracted_comments(self._gpo_message))
         elif origin == "translator":
             comments = gpo.po_message_comments(self._gpo_message)
         elif origin in ["programmer", "developer", "source code"]:
@@ -380,7 +467,7 @@ class pounit(pocommon.pounit):
         if comments and get_libgettextpo_version() < (0, 17, 0):
             comments = "\n".join([line for line in comments.split("\n")])
         # Let's drop the last newline
-        return comments[:-1].decode(self.CPO_ENC)
+        return gpo_decode(comments[:-1])
 
     def addnote(self, text, origin=None, position="append"):
         # ignore empty strings and strings without non-space characters
@@ -414,14 +501,14 @@ class pounit(pocommon.pounit):
                     newlines.append(" " + line)
                 else:
                     newlines.append(line)
-            newnotes = "\n".join(newlines).encode(self.CPO_ENC)
+            newnotes = gpo_encode("\n".join(newlines))
             if origin in ["programmer", "developer", "source code"]:
                 gpo.po_message_set_extracted_comments(self._gpo_message, newnotes)
             else:
                 gpo.po_message_set_comments(self._gpo_message, newnotes)
 
-    def removenotes(self):
-        gpo.po_message_set_comments(self._gpo_message, "")
+    def removenotes(self, origin=None):
+        gpo.po_message_set_comments(self._gpo_message, b"")
 
     def copy(self):
         newpo = self.__class__()
@@ -436,7 +523,7 @@ class pounit(pocommon.pounit):
         """
 
         if not isinstance(otherpo, pounit):
-            super(pounit, self).merge(otherpo, overwrite, comments)
+            super().merge(otherpo, overwrite, comments)
             return
         if comments:
             self.addnote(otherpo.getnotes("translator"), origin="translator", position="merge")
@@ -464,7 +551,7 @@ class pounit(pocommon.pounit):
                 self.markfuzzy()
 
     def isheader(self):
-        #return self.source == u"" and self.target != u""
+        #return self.source == "" and self.target != ""
         # we really want to make sure that there is no msgidcomment or msgctxt
         return self.getid() == "" and len(self.target) > 0
 
@@ -472,10 +559,10 @@ class pounit(pocommon.pounit):
         return len(self.source) == len(self.target) == len(self.getcontext()) == 0
 
     def hastypecomment(self, typecomment):
-        return gpo.po_message_is_format(self._gpo_message, typecomment)
+        return gpo.po_message_is_format(self._gpo_message, gpo_encode(typecomment))
 
     def settypecomment(self, typecomment, present=True):
-        gpo.po_message_set_format(self._gpo_message, typecomment, present)
+        gpo.po_message_set_format(self._gpo_message, gpo_encode(typecomment), present)
 
     def hasmarkedcomment(self, commentmarker):
         commentmarker = "(%s)" % commentmarker
@@ -510,32 +597,32 @@ class pounit(pocommon.pounit):
         :return: Returns the extracted msgidcomments found in this unit's msgid.
         """
         if not text:
-            text = (gpo.po_message_msgid(self._gpo_message) or "").decode(self.CPO_ENC)
+            text = gpo_decode(gpo.po_message_msgid(self._gpo_message)) or ""
         if text:
             return pocommon.extract_msgid_comment(text)
-        return u""
+        return ""
 
     def setmsgidcomment(self, msgidcomment):
         if msgidcomment:
-            self.source = u"_: %s\n%s" % (msgidcomment, self.source)
+            self.source = "_: %s\n%s" % (msgidcomment, self.source)
     msgidcomment = property(_extract_msgidcomments, setmsgidcomment)
 
     def __str__(self):
         pf = pofile(noheader=True)
         pf.addunit(self)
-        return str(pf)
+        return bytes(pf).decode(self.CPO_ENC)
 
     def getlocations(self):
         locations = []
         i = 0
         location = gpo.po_message_filepos(self._gpo_message, i)
         while location:
-            locname = gpo.po_filepos_file(location).decode(self.CPO_ENC)
+            locname = gpo_decode(gpo.po_filepos_file(location))
             locline = gpo.po_filepos_start_line(location)
             if locline == -1:
                 locstring = locname
             else:
-                locstring = locname + u":" + unicode(locline)
+                locstring = ":".join([locname, str(locline)])
             locations.append(pocommon.unquote_plus(locstring))
             i += 1
             location = gpo.po_message_filepos(self._gpo_message, i)
@@ -551,24 +638,25 @@ class pounit(pocommon.pounit):
         else:
             file = location
             line = -1
-        gpo.po_message_add_filepos(self._gpo_message, file, line)
+        gpo.po_message_add_filepos(self._gpo_message, gpo_encode(file), line)
 
     def getcontext(self):
         msgctxt = gpo.po_message_msgctxt(self._gpo_message)
         if msgctxt:
-            return msgctxt.decode(self.CPO_ENC)
+            return gpo_decode(msgctxt)
         else:
             msgidcomment = self._extract_msgidcomments()
             return msgidcomment
 
     def setcontext(self, context):
         context = data.forceunicode(context)
-        gpo.po_message_set_msgctxt(self._gpo_message, context.encode(self.CPO_ENC))
+        gpo.po_message_set_msgctxt(self._gpo_message, gpo_encode(context))
 
     @classmethod
     def buildfromunit(cls, unit, encoding=None):
         """Build a native unit from a foreign unit, preserving as much
-        information as possible."""
+        information as possible.
+        """
         if type(unit) == cls and hasattr(unit, "copy") and callable(unit.copy):
             return unit.copy()
         elif isinstance(unit, pocommon.pounit):
@@ -605,25 +693,26 @@ class pounit(pocommon.pounit):
 class pofile(pocommon.pofile):
     UnitClass = pounit
 
-    def __init__(self, inputfile=None, encoding=None, unitclass=pounit, noheader=False):
+    def __init__(self, inputfile=None, noheader=False, **kwargs):
         self._gpo_memory_file = None
         self._gpo_message_iterator = None
-        self.units = []
         self.sourcelanguage = None
         self.targetlanguage = None
-        self._encoding = 'utf-8'
         if inputfile is None:
+            self.units = []
+            self._encoding = kwargs.get('encoding')
             self._gpo_memory_file = gpo.po_file_create()
             self._gpo_message_iterator = gpo.po_message_iterator(self._gpo_memory_file, None)
-            if not noheader:
-                self.init_headers()
-        else:
-            super(pofile, self).__init__(inputfile=inputfile, encoding=encoding)
+        super().__init__(inputfile=inputfile, noheader=noheader, **kwargs)
 
     def addunit(self, unit, new=True):
         if new:
             gpo.po_message_insert(self._gpo_message_iterator, unit._gpo_message)
-        super(pofile, self).addunit(unit)
+        super().addunit(unit)
+
+    def removeunit(self, unit):
+        # There seems to be no API to remove a message
+        raise ValueError("Unit removal not supported by cpo")
 
     def _insert_header(self, header):
         header._store = self
@@ -665,7 +754,15 @@ class pofile(pocommon.pofile):
                         origpo.setcontext(" ".join(origpo.getlocations()))
                         markedpos.append(thepo)
                     thepo.setcontext(" ".join(thepo.getlocations()))
-                    uniqueunits.append(thepo)
+                    thepo_msgctxt = gpo.po_message_msgctxt(thepo._gpo_message)
+                    idpo_msgctxt = gpo.po_message_msgctxt(id_dict[id]._gpo_message)
+                    if not thepo_msgctxt == idpo_msgctxt:
+                        uniqueunits.append(thepo)
+                    else:
+                        logger.warning(
+                            "Duplicate unit found with msgctx of '%s' and source '%s'",
+                            thepo_msgctxt,
+                            thepo.source)
             else:
                 if not id:
                     if duplicatestyle == "merge":
@@ -683,7 +780,7 @@ class pofile(pocommon.pofile):
         self._gpo_memory_file = new_gpo_memory_file
         self.units = uniqueunits
 
-    def __str__(self):
+    def serialize(self, out):
 
         def obsolete_workaround():
             # Remove all items that are not output by msgmerge when a unit is obsolete.  This is a work
@@ -691,16 +788,17 @@ class pofile(pocommon.pofile):
             # FIXME Do version test in case they fix this bug
             for unit in self.units:
                 if unit.isobsolete():
-                    gpo.po_message_set_extracted_comments(unit._gpo_message, "")
+                    gpo.po_message_set_extracted_comments(unit._gpo_message, b"")
                     location = gpo.po_message_filepos(unit._gpo_message, 0)
                     while location:
                         gpo.po_message_remove_filepos(unit._gpo_message, 0)
                         location = gpo.po_message_filepos(unit._gpo_message, 0)
 
         def writefile(filename):
-            self._gpo_memory_file = gpo.po_file_write_v2(self._gpo_memory_file, filename, xerror_handler)
-            with open(filename) as tfile:
-                return tfile.read()
+            self._gpo_memory_file = gpo.po_file_write_v2(self._gpo_memory_file, gpo_encode(filename), xerror_handler)
+            with open(filename, 'rb') as tfile:
+                content = tfile.read()
+            return content
 
         outputstring = ""
         if self._gpo_memory_file:
@@ -708,16 +806,16 @@ class pofile(pocommon.pofile):
             f, fname = tempfile.mkstemp(prefix='translate', suffix='.po')
             os.close(f)
             outputstring = writefile(fname)
-            if self._encoding != pounit.CPO_ENC:
+            if self.encoding != pounit.CPO_ENC:
                 try:
-                    outputstring = outputstring.decode(pounit.CPO_ENC).encode(self._encoding)
+                    outputstring = outputstring.decode(pounit.CPO_ENC).encode(self.encoding)
                 except UnicodeEncodeError:
-                    self._encoding = pounit.CPO_ENC
+                    self.encoding = pounit.CPO_ENC
                     self.updateheader(content_type="text/plain; charset=UTF-8",
                                       content_transfer_encoding="8bit")
                     outputstring = writefile(fname)
             os.remove(fname)
-        return outputstring
+        out.write(outputstring)
 
     def isempty(self):
         """Returns True if the object doesn't contain any translation units."""
@@ -753,7 +851,7 @@ class pofile(pocommon.pofile):
             input = fname
             os.close(fd)
 
-        self._gpo_memory_file = gpo.po_file_read_v3(input, xerror_handler)
+        self._gpo_memory_file = gpo.po_file_read_v3(gpo_encode(input), xerror_handler)
         if self._gpo_memory_file is None:
             logger.error("Error:")
 
@@ -764,14 +862,14 @@ class pofile(pocommon.pofile):
         # Handle xerrors here
         self._header = gpo.po_file_domain_header(self._gpo_memory_file, None)
         if self._header:
-            charset = gpo.po_header_field(self._header, "Content-Type")
+            charset = gpo_decode(gpo.po_header_field(self._header, gpo_encode("Content-Type")))
             if charset:
                 charset = re.search("charset=([^\\s]+)", charset).group(1)
-            self._encoding = encodingToUse(charset)
+            self.encoding = charset
         self._gpo_message_iterator = gpo.po_message_iterator(self._gpo_memory_file, None)
         newmessage = gpo.po_next_message(self._gpo_message_iterator)
         while newmessage:
-            newunit = pounit(gpo_message=newmessage, encoding=self._encoding)
+            newunit = pounit(gpo_message=newmessage, encoding=self.encoding)
             self.addunit(newunit, new=False)
             newmessage = gpo.po_next_message(self._gpo_message_iterator)
         self._free_iterator()
