@@ -1,4 +1,3 @@
-# -*- coding: utf-8 -*-
 #
 # Copyright 2002-2011 Zuza Software Foundation
 #
@@ -29,14 +28,17 @@ directly, but can be used once cpo has been established to work.
 # - accept only unicodes everywhere
 
 import copy
+import logging
 import re
-import six
 
 from translate.lang import data
 from translate.misc.multistring import multistring
 from translate.storage import base, cpo, pocommon
 
 
+logger = logging.getLogger(__name__)
+
+
 lsep = " "
 """Separator for #: entries"""
 
@@ -47,7 +49,6 @@ msgstr ""
 '''
 
 
-@six.python_2_unicode_compatible
 class pounit(pocommon.pounit):
     # othercomments = []      #   # this is another comment
     # automaticcomments = []  #   #. comment extracted from the source code
@@ -56,7 +57,7 @@ class pounit(pocommon.pounit):
     # prev_msgid = []         #
     # prev_msgid_plural = []  #
     # typecomments = []       #   #, fuzzy
-    # msgidcomment = u""      #   _: within msgid
+    # msgidcomment = ""      #   _: within msgid
     # msgctxt
     # msgid = []
     # msgstr = []
@@ -66,11 +67,11 @@ class pounit(pocommon.pounit):
     __shallow__ = ['_store']
 
     def __init__(self, source=None, **kwargs):
-        pocommon.pounit.__init__(self, source)
+        super().__init__(source)
         self._initallcomments(blankall=True)
-        self._msgctxt = u""
+        self._msgctxt = ""
 
-        self.target = u""
+        self.target = ""
 
     def _initallcomments(self, blankall=False):
         """Initialises allcomments"""
@@ -79,58 +80,56 @@ class pounit(pocommon.pounit):
             self.automaticcomments = []
             self.sourcecomments = []
             self.typecomments = []
-            self.msgidcomment = u""
+            self.msgidcomment = ""
 
-    def getsource(self):
+    @property
+    def source(self):
         return self._source
 
-    def setsource(self, source):
+    @source.setter
+    def source(self, source):
         self._rich_source = None
-#        assert isinstance(source, unicode)
-        source = data.forceunicode(source or u"")
-        source = source or u""
+        source = data.forceunicode(source or "")
+        source = source or ""
         if isinstance(source, multistring):
             self._source = source
-        elif isinstance(source, six.text_type):
+        elif isinstance(source, str):
             self._source = source
-        else:
-            #unicode, list, dict
+        else:  # If it is unicode, list or dict.
             self._source = multistring(source)
-    source = property(getsource, setsource)
 
-    def gettarget(self):
+    @property
+    def target(self):
         """Returns the unescaped msgstr"""
         return self._target
 
-    def settarget(self, target):
+    @target.setter
+    def target(self, target):
         """Sets the msgstr to the given (unescaped) value"""
         self._rich_target = None
-#        assert isinstance(target, unicode)
-#        target = data.forceunicode(target)
         if self.hasplural():
             if isinstance(target, multistring):
                 self._target = target
-            else:
-                #unicode, list, dict
+            else:  # If it is unicode, list or dict.
                 self._target = multistring(target)
         elif isinstance(target, (dict, list)):
             if len(target) == 1:
                 self._target = target[0]
             else:
-                raise ValueError("po msgid element has no plural but msgstr has %d elements (%s)" % (len(target), target))
+                raise ValueError("po msgid element has no plural but msgstr"
+                                 "has %d elements (%s)" % (len(target), target))
         else:
             self._target = target
-    target = property(gettarget, settarget)
 
     def getnotes(self, origin=None):
         """Return comments based on origin value (programmer, developer, source code and translator)"""
         if origin is None:
-            comments = u"\n".join(self.othercomments)
-            comments += u"\n".join(self.automaticcomments)
+            comments = "\n".join(self.othercomments)
+            comments += "\n".join(self.automaticcomments)
         elif origin == "translator":
-            comments = u"\n".join(self.othercomments)
+            comments = "\n".join(self.othercomments)
         elif origin in ["programmer", "developer", "source code"]:
-            comments = u"\n".join(self.automaticcomments)
+            comments = "\n".join(self.automaticcomments)
         else:
             raise ValueError("Comment type not valid")
         return comments
@@ -146,9 +145,9 @@ class pounit(pocommon.pounit):
         if origin in ["programmer", "developer", "source code"]:
             autocomments = True
             commentlist = self.automaticcomments
-        if text.endswith(u'\n'):
+        if text.endswith('\n'):
             text = text[:-1]
-        newcomments = text.split(u"\n")
+        newcomments = text.split("\n")
         if position == "append":
             newcomments = commentlist + newcomments
         elif position == "prepend":
@@ -159,7 +158,7 @@ class pounit(pocommon.pounit):
         else:
             self.othercomments = newcomments
 
-    def removenotes(self):
+    def removenotes(self, origin=None):
         """Remove all the translator's notes (other comments)"""
         self.othercomments = []
 
@@ -170,7 +169,7 @@ class pounit(pocommon.pounit):
         # self.__shallow__
         shallow = set(self.__shallow__)
         # Make deep copies of all members which are not in shallow
-        for key, value in six.iteritems(self.__dict__):
+        for key, value in self.__dict__.items():
             if key not in shallow:
                 setattr(new_unit, key, copy.deepcopy(value))
         # Make shallow copies of all members which are in shallow
@@ -206,7 +205,7 @@ class pounit(pocommon.pounit):
 
         def mergelists(list1, list2, split=False):
             # Decode where necessary (either all bytestrings or all unicode)
-            if six.text_type in [type(item) for item in list2] + [type(item) for item in list1]:
+            if str in [type(item) for item in list2] + [type(item) for item in list1]:
                 for position, item in enumerate(list1):
                     if isinstance(item, bytes):
                         list1[position] = item.decode("utf-8")
@@ -242,7 +241,7 @@ class pounit(pocommon.pounit):
                             list1.append(item)
 
         if not isinstance(otherpo, pounit):
-            super(pounit, self).merge(otherpo, overwrite, comments)
+            super().merge(otherpo, overwrite, comments)
             return
         if comments:
             mergelists(self.othercomments, otherpo.othercomments)
@@ -304,7 +303,7 @@ class pounit(pocommon.pounit):
                 self.typecomments = filter(lambda tcline: tcline.strip() != "#,", typecomments)
 
     def istranslated(self):
-        return super(pounit, self).istranslated() and not self.isobsolete()
+        return super().istranslated() and not self.isobsolete()
 
     def istranslatable(self):
         return not (self.isheader() or self.isblank() or self.isobsolete())
@@ -319,7 +318,7 @@ class pounit(pocommon.pounit):
         """Makes this unit obsolete"""
         self.sourcecomments = []
         self.automaticcomments = []
-        super(pounit, self).makeobsolete()
+        super().makeobsolete()
 
     def hasplural(self):
         """returns whether this pounit contains plural strings..."""
@@ -365,7 +364,7 @@ class pounit(pocommon.pounit):
         return self._msgctxt + self.msgidcomment
 
     def setcontext(self, context):
-        context = data.forceunicode(context or u"")
+        context = data.forceunicode(context or "")
         self._msgctxt = context
 
     def getid(self):
@@ -378,9 +377,9 @@ class pounit(pocommon.pounit):
 #        id = '\0'.join(self.source.strings)
         id = self.source
         if self.msgidcomment:
-            id = u"_: %s\n%s" % (context, id)
+            id = "_: %s\n%s" % (context, id)
         elif context:
-            id = u"%s\04%s" % (context, id)
+            id = "%s\04%s" % (context, id)
         return id
 
     @classmethod
@@ -494,13 +493,19 @@ class pofile(pocommon.pofile):
                         origpo._msgctxt += " ".join(origpo.getlocations())
                         markedpos.append(thepo)
                     thepo._msgctxt += " ".join(thepo.getlocations())
-                    uniqueunits.append(thepo)
+                    if not thepo._msgctxt == id_dict[id]._msgctxt:
+                        uniqueunits.append(thepo)
+                    else:
+                        logger.warning(
+                            "Duplicate unit found with msgctx of '%s' and source '%s'",
+                            thepo._msgctxt,
+                            thepo.source)
             else:
                 if not id:
                     if duplicatestyle == "merge":
                         addcomment(thepo)
                     else:
-                        thepo._msgctxt += u" ".join(thepo.getlocations())
+                        thepo._msgctxt += " ".join(thepo.getlocations())
                 id_dict[id] = thepo
                 uniqueunits.append(thepo)
         self.units = uniqueunits
@@ -510,7 +515,7 @@ class pofile(pocommon.pofile):
         self._cpo_store = cpo.pofile(encoding=self.encoding, noheader=True)
         try:
             self._build_cpo_from_self()
-        except UnicodeEncodeError as e:
+        except UnicodeEncodeError:
             self.encoding = "utf-8"
             self.updateheader(add=True, Content_Type="text/plain; charset=UTF-8")
             self._build_cpo_from_self()
