@@ -1,4 +1,3 @@
-# -*- coding: utf-8 -*-
 #
 # Copyright 2007,2009 Zuza Software Foundation
 #
@@ -30,15 +29,15 @@ b : a string
 """
 
 import re
-import sys
-from io import StringIO, BytesIO
+from io import StringIO
+
+from translate.storage import base
+
 
 try:
     from iniparse import INIConfig
 except ImportError:
-    INIConfig = None
-
-from translate.storage import base
+    raise ImportError("Missing iniparse library.")
 
 
 dialects = {}
@@ -50,7 +49,7 @@ def register_dialect(dialect):
     return dialect
 
 
-class Dialect(object):
+class Dialect:
     """Base class for differentiating dialect options and functions"""
 
     name = None
@@ -85,7 +84,7 @@ class iniunit(base.TranslationUnit):
         self.location = ""
         if source:
             self.source = source
-        super(iniunit, self).__init__(source)
+        super().__init__(source)
 
     def addlocation(self, location):
         self.location = location
@@ -101,11 +100,8 @@ class inifile(base.TranslationStore):
 
     def __init__(self, inputfile=None, dialect="default", **kwargs):
         """construct an INI file, optionally reading in from inputfile."""
-        if INIConfig is None:
-            raise NotImplementedError("Missing iniparse library.")
-
         self._dialect = dialects.get(dialect, DialectDefault)()  # fail correctly/use getattr/
-        super(inifile, self).__init__(**kwargs)
+        super().__init__(**kwargs)
         self.filename = ''
         self._inifile = None
         if inputfile is not None:
@@ -117,14 +113,9 @@ class inifile(base.TranslationStore):
             for location in unit.getlocations():
                 match = re.match('\\[(?P<section>.+)\\](?P<entry>.+)', location)
                 value = self._dialect.escape(unit.target)
-                if sys.version_info[0] == 2:
-                    value = value.encode('utf-8')
                 _outinifile[match.groupdict()['section']][match.groupdict()['entry']] = value
         if _outinifile:
-            if sys.version_info[0] == 3:
-                out.write(str(_outinifile).encode('utf-8'))
-            else:
-                out.write(str(_outinifile))
+            out.write(str(_outinifile).encode('utf-8'))
 
     def parse(self, input):
         """Parse the given file or file source string."""
@@ -138,10 +129,7 @@ class inifile(base.TranslationStore):
             input = inisrc
 
         if isinstance(input, bytes):
-            if sys.version_info[0] == 3:
-                input = StringIO(input.decode('utf-8'))
-            else:
-                input = BytesIO(input)
+            input = StringIO(input.decode('utf-8'))
             self._inifile = INIConfig(input, optionxformvalue=None)
         else:
             self._inifile = INIConfig(open(input), optionxformvalue=None)
