@@ -1,4 +1,3 @@
-# -*- coding: utf-8 -*-
 #
 # Copyright 2007,2009-2011 Zuza Software Foundation
 #
@@ -71,33 +70,36 @@ import json
 import uuid
 from collections import OrderedDict
 
-import six
-
-from translate.misc.deprecation import deprecated
+from translate.lang.data import cldr_plural_categories, plural_tags
 from translate.misc.multistring import multistring
 from translate.storage import base
 
 
-class JsonUnit(base.TranslationUnit):
+class JsonUnit(base.DictUnit):
     """A JSON entry"""
 
+    ID_FORMAT = ".{}"
+
     def __init__(self, source=None, item=None, notes=None, placeholders=None, **kwargs):
-        identifier = str(uuid.uuid4())
+        if source:
+            identifier = hex(hash(source))
+        else:
+            identifier = str(uuid.uuid4())
         # Global identifier across file
-        self._id = '.' + identifier
+        self._id = self.ID_FORMAT.format(identifier)
         # Identifier at this level
         self._item = identifier if item is None else item
         # Type conversion for the unit
-        self._type = six.text_type if source is None else type(source)
+        self._type = str if source is None else type(source)
         if notes:
             self.notes = notes
         self.placeholders = placeholders
         if source:
-            if issubclass(self._type, six.string_types):
+            if issubclass(self._type, str):
                 self.target = source
             else:
                 self.target = str(source)
-        super(JsonUnit, self).__init__(source)
+        super().__init__(source)
 
     @property
     def source(self):
@@ -107,11 +109,6 @@ class JsonUnit(base.TranslationUnit):
     def source(self, source):
         self.target = source
 
-    # Deprecated on 2.3.1
-    @deprecated("Use `source` property instead")
-    def getsource(self):
-        return self.source
-
     def setid(self, value):
         self._id = value
 
@@ -125,31 +122,27 @@ class JsonUnit(base.TranslationUnit):
         """Converts to a string representation."""
         return json.dumps(self.getvalue(), separators=(',', ': '), indent=4, ensure_ascii=False)
 
-    def getkey(self):
-        return self.getid().lstrip('.')
-
     def converttarget(self):
-        if issubclass(self._type, six.string_types):
+        if issubclass(self._type, str):
             return self.target
         else:
             return self._type(self.target)
 
-    def getvalue(self):
-        """Return value to be stored in JSON file."""
-        return {self.getkey(): self.converttarget()}
+    def storevalues(self, output):
+        self.storevalue(output, self.converttarget())
 
 
-class JsonFile(base.TranslationStore):
+class JsonFile(base.DictStore):
     """A JSON file"""
 
     UnitClass = JsonUnit
 
     def __init__(self, inputfile=None, filter=None, **kwargs):
         """construct a JSON file, optionally reading in from inputfile."""
-        super(JsonFile, self).__init__(**kwargs)
+        super().__init__(**kwargs)
         self._filter = filter
         self.filename = ''
-        self._file = u''
+        self._file = ''
         self.dump_args = {
             'separators': (',', ': '),
             'indent': 4,
@@ -159,24 +152,12 @@ class JsonFile(base.TranslationStore):
             self.parse(inputfile)
 
     def serialize(self, out):
-        def merge(d1, d2):
-            for k in d2:
-                if k in d1:
-                    if isinstance(d1[k], dict) and isinstance(d2[k], dict):
-                        merge(d1[k], d2[k])
-                    elif isinstance(d1[k], list) and isinstance(d2[k], list):
-                        d1[k].extend(d2[k])
-                    else:
-                        d1[k] = d2[k]
-                else:
-                    d1[k] = d2[k]
         units = OrderedDict()
-        for unit in self.unit_iter():
-            merge(units, unit.getvalue())
+        self.serialize_units(units)
         out.write(json.dumps(units, **self.dump_args).encode(self.encoding))
         out.write(b'\n')
 
-    def _extract_units(self, data, stop=None, prev="", name_node=None, name_last_node=None, last_node=None):
+    def _extract_units(self, data, stop=None, prev=None, name_node=None, name_last_node=None, last_node=None):
         """Recursive function to extract items from the data files
 
         :param data: the current branch to walk down
@@ -186,13 +167,15 @@ class JsonFile(base.TranslationStore):
         :param name_last_node: the name of the last node
         :param last_node: the last list or dict
         """
+        if prev is None:
+            prev = self.UnitClass.IdClass([])
         if isinstance(data, dict):
-            for k, v in six.iteritems(data):
-                for x in self._extract_units(v, stop, "%s.%s" % (prev, k), k, None, data):
+            for k, v in data.items():
+                for x in self._extract_units(v, stop, prev + [('key', k)], k, None, data):
                     yield x
         elif isinstance(data, list):
             for i, item in enumerate(data):
-                for x in self._extract_units(item, stop, "%s[%s]" % (prev, i), i, name_node, data):
+                for x in self._extract_units(item, stop, prev + [('index', i)], i, name_node, data):
                     yield x
         # apply filter
         elif (stop is None or
@@ -200,7 +183,7 @@ class JsonFile(base.TranslationStore):
               (isinstance(last_node, list) and name_last_node in stop)):
 
             unit = self.UnitClass(data, name_node)
-            unit.setid(prev)
+            unit.set_unitid(prev)
             yield unit
 
     def parse(self, input):
@@ -214,7 +197,10 @@ class JsonFile(base.TranslationStore):
             input.close()
             input = src
         if isinstance(input, bytes):
-            input = input.decode('utf-8')
+            # The JSON files should be UTF-8, but implementations
+            # that parse JSON texts MAY ignore the presence of a byte order mark
+            # rather than treating it as an error, see RFC7159
+            input, self.encoding = self.detect_encoding(input)
         try:
             self._file = json.loads(input, object_pairs_hook=OrderedDict)
         except ValueError as e:
@@ -226,17 +212,8 @@ class JsonFile(base.TranslationStore):
 
 class JsonNestedUnit(JsonUnit):
 
-    def getkey(self):
-        return self.getid().lstrip('.').split('.')
-
-    def getvalue(self):
-        ret = self.converttarget()
-        for k in reversed(self.getkey()):
-            if '[' in k and k[-1] == ']':
-                k = k.split('[')[0]
-                ret = [ret]
-            ret = OrderedDict({k: ret})
-        return ret
+    def storevalues(self, output):
+        self.storevalue(output, self.converttarget())
 
 
 class JsonNestedFile(JsonFile):
@@ -246,7 +223,7 @@ class JsonNestedFile(JsonFile):
 
 
 class WebExtensionJsonUnit(JsonUnit):
-    def getvalue(self):
+    def storevalues(self, output):
         value = OrderedDict((
             ('message', self.target),
         ))
@@ -254,7 +231,7 @@ class WebExtensionJsonUnit(JsonUnit):
             value['description'] = self.notes
         if self.placeholders:
             value['placeholders'] = self.placeholders
-        return {self.getid(): value}
+        self.storevalue(output, value)
 
 
 class WebExtensionJsonFile(JsonFile):
@@ -268,8 +245,8 @@ class WebExtensionJsonFile(JsonFile):
 
     UnitClass = WebExtensionJsonUnit
 
-    def _extract_units(self, data, stop=None, prev="", name_node=None, name_last_node=None, last_node=None):
-        for item, value in six.iteritems(data):
+    def _extract_units(self, data, stop=None, prev=None, name_node=None, name_last_node=None, last_node=None):
+        for item, value in data.items():
             unit = self.UnitClass(
                 value.get('message', ''),
                 item,
@@ -318,18 +295,12 @@ class I18NextUnit(JsonNestedUnit):
         self._rich_target = None
         self._target = target
 
-    def getvalue(self):
+    def storevalues(self, output):
         if not isinstance(self.target, multistring):
-            return super(I18NextUnit, self).getvalue()
-
-        ret = OrderedDict()
-        for i, value in enumerate(self.target.strings):
-            ret[self._item[i]] = value
-
-        path = self.getid().lstrip('.').split('.')[:-1]
-        for k in reversed(path):
-            ret = {k: ret}
-        return ret
+            super().storevalues(output)
+        else:
+            for i, value in enumerate(self.target.strings):
+                self.storevalue(output, value, override_key=self._item[i])
 
 
 class I18NextFile(JsonNestedFile):
@@ -340,13 +311,15 @@ class I18NextFile(JsonNestedFile):
 
     UnitClass = I18NextUnit
 
-    def _extract_units(self, data, stop=None, prev="", name_node=None, name_last_node=None, last_node=None):
+    def _extract_units(self, data, stop=None, prev=None, name_node=None, name_last_node=None, last_node=None):
+        if prev is None:
+            prev = self.UnitClass.IdClass([])
         if isinstance(data, dict):
             plurals_multiple = [key.rsplit('_', 1)[0] for key in data if key.endswith('_0')]
             plurals_simple = [key.rsplit('_', 1)[0] for key in data if key.endswith('_plural')]
             processed = set()
 
-            for k, v in six.iteritems(data):
+            for k, v in data.items():
                 # Check already processed items
                 if k in processed:
                     continue
@@ -374,15 +347,143 @@ class I18NextFile(JsonNestedFile):
                         sources.append(data[key])
                         items.append(key)
                     unit = self.UnitClass(multistring(sources), items)
-                    unit.setid("%s.%s" % (prev, plural_base))
+                    newid = prev + [('key', plural_base)]
+                    unit.set_unitid(newid)
                     yield unit
                     continue
 
-                for x in self._extract_units(v, stop, "%s.%s" % (prev, k), k, None, data):
+                for x in self._extract_units(v, stop, prev + [('key', k)], k, None, data):
                     yield x
         else:
-            parent = super(I18NextFile, self)._extract_units(
+            parent = super()._extract_units(
                 data, stop, prev, name_node, name_last_node, last_node
             )
             for x in parent:
                 yield x
+
+
+class GoI18NJsonUnit(JsonUnit):
+    ID_FORMAT = "{}"
+
+    def getvalue(self):
+        target = self.target
+        if isinstance(target, multistring):
+            strings = list(target.strings)
+            if len(self._store.plural_tags) > len(target.strings):
+                strings += [""] * (len(self._store.plural_tags) - len(target.strings))
+            target = OrderedDict([
+                (plural, strings[offset]) for offset, plural in enumerate(self._store.plural_tags)
+            ])
+        value = OrderedDict((
+            ('id', self.getid()),
+        ))
+        if self.notes:
+            value['description'] = self.notes
+        value['translation'] = target
+        return value
+
+
+class GoI18NJsonFile(JsonFile):
+    """go-i18n JSON file
+
+    See following URLs for doc:
+
+    https://github.com/nicksnyder/go-i18n
+    https://godoc.org/github.com/nicksnyder/go-i18n/v2
+    """
+
+    UnitClass = GoI18NJsonUnit
+
+    @property
+    def plural_tags(self):
+        locale = self.gettargetlanguage()
+        if locale:
+            locale = locale.replace('_', '-').split('-')[0]
+        else:
+            locale = "en"
+        return plural_tags.get(locale, plural_tags['en'])
+
+    def _extract_units(self, data, stop=None, prev=None, name_node=None, name_last_node=None, last_node=None):
+        for value in data:
+            translation = value.get('translation', '')
+            if isinstance(translation, dict):
+                # Ordered list of plurals
+                translation = multistring(
+                    [translation.get(key) for key in cldr_plural_categories if key in translation]
+                )
+            unit = self.UnitClass(
+                translation,
+                value.get('id', ''),
+                value.get('description', ''),
+            )
+            unit.setid(value.get('id', ''))
+            yield unit
+
+    def serialize(self, out):
+        units = [unit.getvalue() for unit in self.units]
+        out.write(json.dumps(units, **self.dump_args).encode(self.encoding))
+        out.write(b'\n')
+
+
+class ARBJsonUnit(JsonUnit):
+    ID_FORMAT = "{}"
+
+    def __init__(self, source=None, item=None, notes=None, placeholders=None, metadata=None, **kwargs):
+        super().__init__(source, item, notes, placeholders, **kwargs)
+        self.metadata = metadata or {}
+
+    def storevalues(self, output):
+        if self.notes:
+            self.metadata['description'] = self.notes
+        identifier = self.getid()
+        if identifier == "@":
+            for key, value in self.metadata.items():
+                self.storevalue(output, value, override_key=key)
+        else:
+            self.storevalue(output, self.target, override_key=identifier)
+            self.storevalue(output, self.metadata, override_key='@{}'.format(identifier))
+
+    def isheader(self):
+        return self._id == "@"
+
+
+class ARBJsonFile(JsonFile):
+    """ARB JSON file
+
+    See following URLs for doc:
+
+    https://github.com/google/app-resource-bundle/wiki/ApplicationResourceBundleSpecification
+    https://flutter.dev/docs/development/accessibility-and-localization/internationalization#appendix-using-the-dart-intl-tools
+    """
+
+    UnitClass = ARBJsonUnit
+
+    def __init__(self, inputfile=None, filter=None, **kwargs):
+        super().__init__(inputfile, filter, **kwargs)
+        self.dump_args = {
+            'separators': (',', ': '),
+            'indent': 2,
+            'ensure_ascii': False,
+        }
+
+    def _extract_units(self, data, stop=None, prev=None, name_node=None, name_last_node=None, last_node=None):
+        # Extract metadata as header
+        metadata = OrderedDict([(key, value) for key, value in data.items() if key.startswith("@@")])
+        if metadata:
+            unit = self.UnitClass(metadata=metadata)
+            unit.setid("@")
+            yield unit
+
+        for item, value in data.items():
+            if item.startswith("@"):
+                continue
+            metadata = data.get("@{}".format(item), {})
+            unit = self.UnitClass(
+                value,
+                item,
+                metadata.get('description', ''),
+                metadata.get('placeholders', None),
+                metadata=metadata,
+            )
+            unit.setid(item)
+            yield unit
