@@ -1,4 +1,3 @@
-# -*- coding: utf-8 -*-
 #
 # Copyright 2008-2009 Zuza Software Foundation
 #
@@ -44,14 +43,11 @@ The placeables model follows the XLIFF standard's list of placeables.
 Please refer to the XLIFF specification to get a better understanding.
 """
 
-from . import base
-from . import interfaces
-from . import general
-from . import xliff
+from . import base, general, interfaces, xliff
 from .base import *
 from .base import __all__ as all_your_base
-from .strelem import StringElem
 from .parse import parse
+from .strelem import StringElem
 
 
 __all__ = (
