@@ -1,4 +1,3 @@
-# -*- coding: utf-8 -*-
 #
 # Copyright 2009 Zuza Software Foundation
 #
@@ -17,7 +16,6 @@
 # You should have received a copy of the GNU General Public License
 # along with this program; if not, see <http://www.gnu.org/licenses/>.
 
-import six
 from io import BytesIO
 
 from translate.search.match import terminologymatcher
@@ -28,7 +26,7 @@ from translate.storage.pypo import pofile
 
 
 class TestTerminologyPlaceable:
-    TERMINOLOGY = u"""
+    TERMINOLOGY = """
 msgid "name"
 msgstr "naam"
 
@@ -45,7 +43,7 @@ msgstr "lêernaam"
     def setup_method(self, method):
         self.term_po = pofile(BytesIO(self.TERMINOLOGY.encode('utf-8')))
         self.matcher = terminologymatcher(self.term_po)
-        self.test_string = u'<b>Inpüt</b> file name thingy.'
+        self.test_string = '<b>Inpüt</b> file name thingy.'
 
     def test_simple_terminology(self):
         TerminologyPlaceable.matchers = [self.matcher]
@@ -58,5 +56,5 @@ msgstr "lêernaam"
         term = tree.sub[3].sub[1]
 
         assert isinstance(term, TerminologyPlaceable)
-        assert six.text_type(term) == self.term_po.getunits()[2].source
-        assert term.translate() == six.text_type(self.term_po.getunits()[2].target)
+        assert str(term) == self.term_po.getunits()[2].source
+        assert term.translate() == str(self.term_po.getunits()[2].target)
