@@ -1,11 +1,9 @@
-# -*- coding: utf-8 -*-
-
-from __future__ import print_function, unicode_literals
 
+import pytest
 from lxml import etree
 
-from translate.storage import aresource, test_monolingual
 from translate.misc.multistring import multistring
+from translate.storage import aresource, test_monolingual
 from translate.storage.base import TranslationStore
 
 
@@ -179,13 +177,13 @@ class TestAndroidResourceUnit(test_monolingual.TestMonolingualUnit):
 
     def test_escape_html_double_space(self):
         string = '<b>html code \'to  escape\'</b> some \'here\''
-        xml = ('<string name="teststring"><b>"html code \\\'to  escape\\\'"</b> some \\\'here\\\''
+        xml = ('<string name="teststring"><b>html code \\\'to \\u0020escape\\\'</b> some \\\'here\\\''
                '</string>\n')
         self.__check_escape(string, xml)
 
     def test_escape_html_deep_double_space(self):
         string = '<b>html code \'to  <i>escape</i>\'</b> some \'here\''
-        xml = ('<string name="teststring"><b>"html code \\\'to  "<i>escape</i>\\\'</b> some \\\'here\\\''
+        xml = ('<string name="teststring"><b>html code \\\'to \\u0020<i>escape</i>\\\'</b> some \\\'here\\\''
                '</string>\n')
         self.__check_escape(string, xml)
 
@@ -196,6 +194,15 @@ class TestAndroidResourceUnit(test_monolingual.TestMonolingualUnit):
                '</string>\n')
         self.__check_escape(string, xml)
 
+    def test_escape_quoted_newlines(self):
+        self.__check_escape(
+            "\n\nstring with newlines",
+            r"""<string name="teststring">"
+\n
+\nstring with newlines"</string>
+"""
+        )
+
     ############################ Check string parse ###########################
 
     def test_parse_message_with_newline(self):
@@ -204,7 +211,7 @@ class TestAndroidResourceUnit(test_monolingual.TestMonolingualUnit):
         self.__check_parse(string, xml)
 
     def test_parse_message_with_newline_in_xml(self):
-        string = 'message \nwith\n newline\n in xml'
+        string = 'message\nwith\n newline\n in xml'
         xml = ('<string name="teststring">message\n\\nwith\\n\nnewline\\n\nin xml'
                '</string>\n')
         self.__check_parse(string, xml)
@@ -234,6 +241,15 @@ class TestAndroidResourceUnit(test_monolingual.TestMonolingualUnit):
         xml = '<string name="teststring">" leading space"</string>\n'
         self.__check_parse(string, xml)
 
+    def test_parse_quoted_newlines(self):
+        self.__check_parse(
+            "\n\nstring with newlines",
+            r"""<string name="teststring">"
+\n
+\nstring with newlines"</string>
+"""
+        )
+
     def test_parse_xml_entities(self):
         string = '>xml&entities'
         xml = '<string name="teststring">&gt;xml&amp;entities</string>\n'
@@ -326,6 +342,16 @@ class TestAndroidResourceUnit(test_monolingual.TestMonolingualUnit):
                '</plurals>\n')
         self.__check_parse(mString, xml)
 
+    def test_plural_parse_message_with_comments(self):
+        mString = multistring(['one message', 'other message'])
+        xml = ('<plurals name="teststring">\n'
+               '    <!-- comment of one string -->\n'
+               '    <item quantity="one">one message</item>\n'
+               '    <!-- comment of other string -->\n'
+               '    <item quantity="other">other message</item>\n'
+               '</plurals>\n')
+        self.__check_parse(mString, xml)
+
     def test_parse_html_quote(self):
         string = 'start \'here\' <b>html code \'to escape\'</b> also \'here\''
         xml = ('<string name="teststring">start \\\'here\\\' <b>html code \\\'to escape\\\'</b> also \\\'here\\\''
@@ -364,7 +390,7 @@ class TestAndroidResourceUnit(test_monolingual.TestMonolingualUnit):
 
     def test_parse_html_double_space_quoted(self):
         string = '<b>html code \'to  escape\'</b> some \'here\''
-        xml = ('<string name="teststring"><b>"html code \'to  escape\'"</b>" some \'here\'"'
+        xml = ('<string name="teststring"><b>html code \'to \\u0020escape\'</b> some \'here\''
                '</string>\n')
         self.__check_parse(string, xml)
 
@@ -381,6 +407,11 @@ class TestAndroidResourceUnit(test_monolingual.TestMonolingualUnit):
                '</string>\n')
         self.__check_parse(string, xml)
 
+    def test_parse_unicode(self):
+        with pytest.raises(ValueError):
+            self.__check_parse('', r'<string name="test">\utest</string>')
+        self.__check_parse('\u0230', r'<string name="test">\u0230</string>')
+
 
 class TestAndroidResourceFile(test_monolingual.TestMonolingualStore):
     StoreClass = aresource.AndroidResourceFile
@@ -472,3 +503,128 @@ class TestAndroidResourceFile(test_monolingual.TestMonolingualStore):
         otherstore.parse(other)
         store.addunit(otherstore.units[0], True)
         assert bytes(store) == expected
+
+    def test_entity(self):
+        content = b'''<?xml version="1.0" encoding="utf-8"?>
+<!DOCTYPE resources [
+<!ENTITY appName "Linphone">
+]>
+<resources>
+    <string name="app_name">&appName;</string>
+    <string name="app_core">&appName; Core</string>
+</resources>'''
+        store = self.StoreClass()
+        store.parse(content)
+        assert store.units[0].source == "&appName;"
+        store.units[0].target = "&appName;"
+        assert store.units[1].source == "&appName; Core"
+        store.units[1].target = "&appName; Core"
+        assert bytes(store) == content
+
+    def test_indent(self):
+        content = '''<?xml version="1.0" encoding="utf-8"?>
+<!DOCTYPE resources [
+<!ENTITY url_privacy_policy "http://example.com/">
+]>
+<resources>
+    <string name="privacy_policy"><u><a href="&url_privacy_policy;">Datenschutzerklärung</a></u></string>
+</resources>'''.encode('utf-8')
+        store = self.StoreClass()
+        store.parse(content)
+        assert bytes(store) == content
+
+    def test_edit_plural_markup(self):
+        content = '''<?xml version="1.0" encoding="utf-8"?>
+<resources>
+    <plurals name="teststring">
+        <item quantity="one"><xliff:g xmlns:xliff="urn:oasis:names:tc:xliff:document:1.2" id="count">%d</xliff:g> den</item>
+        <item quantity="few"><xliff:g xmlns:xliff="urn:oasis:names:tc:xliff:document:1.2" id="count">%d</xliff:g> dny</item>
+        <item quantity="other"><xliff:g xmlns:xliff="urn:oasis:names:tc:xliff:document:1.2" id="count">%d</xliff:g> dnu</item>
+    </plurals>
+</resources>'''.encode('utf-8')
+        store = self.StoreClass()
+        store.targetlanguage = 'cs'
+        store.parse(content)
+        store.units[0].target = multistring([
+            '<xliff:g xmlns:xliff="urn:oasis:names:tc:xliff:document:1.2" id="count">%d</xliff:g> den',
+            '<xliff:g xmlns:xliff="urn:oasis:names:tc:xliff:document:1.2" id="count">%d</xliff:g> dny',
+            '<xliff:g xmlns:xliff="urn:oasis:names:tc:xliff:document:1.2" id="count">%d</xliff:g> dnu',
+        ])
+        assert bytes(store) == content
+
+    def test_entity_add(self):
+        content = '''<?xml version="1.0" encoding="utf-8"?>
+<!DOCTYPE resources [
+<!ENTITY appName "Zkouška">
+]>
+<resources>
+    <string name="app_name">&appName;</string>
+    <string name="app_core">&appName; Core</string>
+</resources>'''.encode('utf-8')
+        store = self.StoreClass()
+        store.parse(content)
+        second = self.StoreClass()
+        second.addunit(store.units[0], True)
+        second.addunit(store.units[1], True)
+        store = self.StoreClass()
+        assert bytes(second) == content
+
+    def test_markup_remove(self):
+        template = '''<?xml version="1.0" encoding="utf-8"?>
+<resources>
+    <string name="privacy_policy"><u>Datenschutzerklärung</u></string>
+</resources>'''
+        content = template.encode('utf-8')
+        newcontent = template.replace('<u>', '').replace('</u>', '').encode('utf-8')
+        store = self.StoreClass()
+        store.parse(content)
+        assert bytes(store) == content
+        store.units[0].target = "Datenschutzerklärung"
+        assert bytes(store) == newcontent
+
+    def test_markup_set(self):
+        template = '''<?xml version="1.0" encoding="utf-8"?>
+<resources>
+    <!--Multimedia tab-->
+    <string name="id">Multimedia tab</string>
+</resources>'''
+        content = template.encode('utf-8')
+        newcontent = template.replace('>Multimedia tab<', '>Other <b>tab</b><').encode('utf-8')
+        store = self.StoreClass()
+        store.parse(content)
+        assert bytes(store) == content
+        store.units[0].target = "Other <b>tab</b>"
+        assert bytes(store) == newcontent
+
+    def test_edit_plural_others(self):
+        content = '''<?xml version="1.0" encoding="utf-8"?>
+<resources>
+    <plurals name="teststring">
+        <item quantity="one"><xliff:g xmlns:xliff="urn:oasis:names:tc:xliff:document:1.2" id="count">%d</xliff:g> den</item>
+        <item quantity="few"><xliff:g xmlns:xliff="urn:oasis:names:tc:xliff:document:1.2" id="count">%d</xliff:g> dny</item>
+        <item quantity="many"><xliff:g xmlns:xliff="urn:oasis:names:tc:xliff:document:1.2" id="count">%d</xliff:g> dnu</item>
+        <item quantity="other"><xliff:g xmlns:xliff="urn:oasis:names:tc:xliff:document:1.2" id="count">%d</xliff:g> dnu</item>
+    </plurals>
+</resources>'''.encode('utf-8')
+        store = self.StoreClass()
+        store.targetlanguage = 'ru'
+        store.parse(content)
+        store.units[0].target = multistring([
+            '<xliff:g xmlns:xliff="urn:oasis:names:tc:xliff:document:1.2" id="count">%d</xliff:g> den',
+            '<xliff:g xmlns:xliff="urn:oasis:names:tc:xliff:document:1.2" id="count">%d</xliff:g> dny',
+            '<xliff:g xmlns:xliff="urn:oasis:names:tc:xliff:document:1.2" id="count">%d</xliff:g> dnu',
+        ])
+        assert bytes(store) == content
+
+    def test_markup_quotes_set(self):
+        template = '''<?xml version="1.0" encoding="utf-8"?>
+<resources>
+    <string name="id">Test</string>
+</resources>'''
+        content = template.encode()
+        newcontent = template.replace('>Test<', '>Test <b>string</b> with \\u0020space<')
+        store = self.StoreClass()
+        store.parse(content)
+        assert bytes(store) == content
+        store.units[0].target = 'Test <b>string</b> with  space'
+        assert bytes(store).decode() == newcontent
