@@ -61,8 +61,7 @@ GENERAL@2|Notes,"cable, motor, switch"
     @mark.xfail(reason="Bug #3356")
     def test_context_is_parsed(self):
         """Tests that units with the same source are different based on context."""
-        source = ('"65066","Ogre","Ogro"\n'
-                  '"65067","Ogre","Ogros"')
+        source = b'"65066","Ogre","Ogro"\n"65067","Ogre","Ogros"'
         store = self.parse_store(source)
         assert len(store.units) == 2
         unit1 = store.units[0]
@@ -82,3 +81,12 @@ GENERAL@2|Notes,"cable, motor, switch"
         assert store.units[0].source == 'te\\nst'
         assert store.units[0].target == 'ot\\nher'
         assert bytes(store) == content
+
+    def test_parse_sample(self):
+        content = b'"location";"source";"target"\r\n"foo.c:1";"te\\nst";"ot\\nher"\r\n'
+        store = self.StoreClass()
+        store.parse(content, sample_length=None)
+        assert len(store.units) == 1
+        assert store.units[0].source == 'te\\nst'
+        assert store.units[0].target == 'ot\\nher'
+        assert bytes(store) == content
