@@ -1,4 +1,3 @@
-# -*- coding: utf-8 -*-
 #
 # Copyright 2015 Zuza Software Foundation
 # Copyright 2015 Sarah Hale
@@ -18,8 +17,9 @@
 # You should have received a copy of the GNU General Public License
 # along with this program; if not, see <http://www.gnu.org/licenses/>.
 
-from translate.misc import wStringIO
-from translate.storage import test_monolingual, resx
+from io import BytesIO
+
+from translate.storage import resx, test_monolingual
 
 
 class TestRESXUnit(test_monolingual.TestMonolingualUnit):
@@ -29,7 +29,7 @@ class TestRESXUnit(test_monolingual.TestMonolingualUnit):
 class TestRESXUnitFromParsedString(TestRESXUnit):
     resxsource = XMLskeleton = '''<?xml version="1.0" encoding="utf-8"?>
 <root>
-  <xsd:schema id="root" xmlns="" xmlns:xsd="http://www.w3.org/2001/XMLSchema" xmlns:msdata="urn:schemas-microsoft-com:xml-msdata">
+  <xsd:schema xmlns="" xmlns:xsd="http://www.w3.org/2001/XMLSchema" xmlns:msdata="urn:schemas-microsoft-com:xml-msdata" id="root">
     <xsd:import namespace="http://www.w3.org/XML/1998/namespace" />
     <xsd:element name="root" msdata:IsDataSet="true">
       <xsd:complexType>
@@ -82,14 +82,42 @@ class TestRESXUnitFromParsedString(TestRESXUnit):
     <value>2.0</value>
   </resheader>
   <data name="key" xml:space="preserve">
-     <value>Test String</value>
+    <value>Test String</value>
   </data>
   %s
 </root>'''
 
     def setup_method(self, method):
-        store = resx.RESXFile.parsestring(self.resxsource)
-        self.unit = store.units[0]
+        self.store = resx.RESXFile.parsestring(self.resxsource % '')
+        self.unit = self.store.units[0]
+
+    def _assert_store(self, expected_resx):
+        output_file = BytesIO()
+        self.store.serialize(output_file)
+        actual_resx = output_file.getvalue().decode("utf-8")
+        assert actual_resx == expected_resx
+
+    def test_newunit(self):
+        new_unit = resx.RESXUnit("New translated value")
+        new_unit.setid("test")
+        self.store.addunit(new_unit)
+
+        expected_resx = self.resxsource % '''<data name="test" xml:space="preserve">
+    <value>New translated value</value>
+  </data>'''
+        self._assert_store(expected_resx)
+
+    def test_newunit_comment(self):
+        new_unit = resx.RESXUnit("New translated value")
+        new_unit.setid("test")
+        new_unit.addnote("this unit didn't exist before")
+        self.store.addunit(new_unit)
+
+        expected_resx = self.resxsource % '''<data name="test" xml:space="preserve">
+    <value>New translated value</value>
+    <comment>this unit didn't exist before</comment>
+  </data>'''
+        self._assert_store(expected_resx)
 
 
 class TestRESXfile(test_monolingual.TestMonolingualStore):
@@ -97,6 +125,6 @@ class TestRESXfile(test_monolingual.TestMonolingualStore):
 
     def resxparse(self, resxsource):
         """helper that parses resx source without requiring files"""
-        dummyfile = wStringIO.StringIO(resxsource)
+        dummyfile = BytesIO(resxsource.encode())
         txtfile = self.StoreClass(dummyfile)
         return txtfile
