@@ -1,5 +1,3 @@
-#!/usr/bin/env python
-# -*- coding: utf-8 -*-
 #
 # Copyright 2008 Zuza Software Foundation
 #
@@ -19,7 +17,6 @@
 # along with this program; if not, see <http://www.gnu.org/licenses/>.
 
 """Import units from translations files into tmdb."""
-from __future__ import print_function
 
 import logging
 import os
